@@ -73,11 +73,15 @@
      VAR_REQUEST_XFR = 289,
      VAR_NOTIFY = 290,
      VAR_PROVIDE_XFR = 291,
-     VAR_KEY = 292,
-     VAR_ALGORITHM = 293,
-     VAR_SECRET = 294,
-     VAR_AXFR = 295,
-     VAR_VERBOSITY = 296
+     VAR_OUTGOING_INTERFACE = 292,
+     VAR_ALLOW_AXFR_FALLBACK = 293,
+     VAR_KEY = 294,
+     VAR_ALGORITHM = 295,
+     VAR_SECRET = 296,
+     VAR_AXFR = 297,
+     VAR_UDP = 298,
+     VAR_VERBOSITY = 299,
+     VAR_HIDE_VERSION = 300
    };
 #endif
 /* Tokens.  */
@@ -115,11 +119,15 @@
 #define VAR_REQUEST_XFR 289
 #define VAR_NOTIFY 290
 #define VAR_PROVIDE_XFR 291
-#define VAR_KEY 292
-#define VAR_ALGORITHM 293
-#define VAR_SECRET 294
-#define VAR_AXFR 295
-#define VAR_VERBOSITY 296
+#define VAR_OUTGOING_INTERFACE 292
+#define VAR_ALLOW_AXFR_FALLBACK 293
+#define VAR_KEY 294
+#define VAR_ALGORITHM 295
+#define VAR_SECRET 296
+#define VAR_AXFR 297
+#define VAR_UDP 298
+#define VAR_VERBOSITY 299
+#define VAR_HIDE_VERSION 300
 
 
 
@@ -131,7 +139,7 @@ typedef union YYSTYPE
 	char*	str;
 }
 /* Line 1489 of yacc.c.  */
-#line 135 "configparser.h"
+#line 143 "configparser.h"
 	YYSTYPE;
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
