@@ -52,10 +52,17 @@ diff_write_packet(const char* zone, uint32_t new_serial, uint16_t id,
 	uint32_t seq_nr, uint8_t* data, size_t len, nsd_options_t* opt)
 {
 	const char* filename = opt->difffile;
+	struct timeval tv;
 	FILE *df;
 	uint32_t file_len = sizeof(uint32_t) + strlen(zone) +
 		sizeof(new_serial) + sizeof(id) + sizeof(seq_nr) + len;
 
+	if (gettimeofday(&tv, NULL) != 0) {
+		log_msg(LOG_ERR, "could not set timestamp for %s: %s",
+			filename, strerror(errno));
+		return;
+	}
+
 	df = fopen(filename, "a");
 	if(!df) {
 		log_msg(LOG_ERR, "could not open file %s for append: %s",
@@ -64,6 +71,8 @@ diff_write_packet(const char* zone, uint32_t new_serial, uint16_t id,
 	}
 
 	if(!write_32(df, DIFF_PART_IXFR) ||
+		!write_32(df, (uint32_t) tv.tv_sec) ||
+		!write_32(df, (uint32_t) tv.tv_usec) ||
 		!write_32(df, file_len) ||
 		!write_str(df, zone) ||
 		!write_32(df, new_serial) ||
@@ -82,13 +91,19 @@ diff_write_packet(const char* zone, uint32_t new_serial, uint16_t id,
 void
 diff_write_commit(const char* zone, uint32_t old_serial,
 	uint32_t new_serial, uint16_t id, uint32_t num_parts,
-	uint8_t commit, const char* log_str,
-	nsd_options_t* opt)
+	uint8_t commit, const char* log_str, nsd_options_t* opt)
 {
 	const char* filename = opt->difffile;
+	struct timeval tv;
 	FILE *df;
 	uint32_t len;
 
+	if (gettimeofday(&tv, NULL) != 0) {
+		log_msg(LOG_ERR, "could not set timestamp for %s: %s",
+			filename, strerror(errno));
+		return;
+	}
+
 	df = fopen(filename, "a");
 	if(!df) {
 		log_msg(LOG_ERR, "could not open file %s for append: %s",
@@ -96,11 +111,13 @@ diff_write_commit(const char* zone, uint32_t old_serial,
 		return;
 	}
 
-	len = strlen(zone)+sizeof(len) + sizeof(old_serial) +
+	len = strlen(zone) + sizeof(len) + sizeof(old_serial) +
 		sizeof(new_serial) + sizeof(id) + sizeof(num_parts) +
-		sizeof(commit) + strlen(log_str)+sizeof(len);
+		sizeof(commit) + strlen(log_str) + sizeof(len);
 
 	if(!write_32(df, DIFF_PART_SURE) ||
+		!write_32(df, (uint32_t) tv.tv_sec) ||
+		!write_32(df, (uint32_t) tv.tv_usec) ||
 		!write_32(df, len) ||
 		!write_str(df, zone) ||
 		!write_32(df, old_serial) ||
@@ -118,6 +135,10 @@ diff_write_commit(const char* zone, uint32_t old_serial,
 	fclose(df);
 }
 
+/*
+ * Checksum to signal no data change occured (for example, by a
+ * zonec run.
+ */
 int
 db_crc_different(namedb_type* db)
 {
@@ -169,12 +190,12 @@ db_crc_different(namedb_type* db)
 int
 diff_read_32(FILE *in, uint32_t* result)
 {
-        if (fread(result, sizeof(*result), 1, in) == 1) {
-                *result = ntohl(*result);
-                return 1;
-        } else {
-                return 0;
-        }
+	if (fread(result, sizeof(*result), 1, in) == 1) {
+		*result = ntohl(*result);
+		return 1;
+	} else {
+		return 0;
+	}
 }
 
 int
@@ -314,7 +335,6 @@ rdatas_equal(rdata_atom_type *a, rdata_atom_type *b, int num, uint16_t type)
 	for(k = 0; k < num; k++)
 	{
 		if(rdata_atom_is_domain(type, k)) {
-			/* check dname: should compare case insensitive */
 			if(dname_compare(domain_dname(a[k].domain),
 				domain_dname(b[k].domain))!=0)
 				return 0;
@@ -378,6 +398,9 @@ delete_RR(namedb_type* db, const dname_type* dname,
 		ssize_t rdata_num;
 		int rrnum;
 		temptable = domain_table_create(temp_region);
+		/* This will ensure that the dnames in rdata are
+		 * normalized, conform RFC 4035, section 6.2
+		 */
 		rdata_num = rdata_wireformat_to_rdata_atoms(
 			temp_region, temptable, type, rdatalen, packet, &rdatas);
 		if(rdata_num == -1) {
@@ -446,6 +469,9 @@ add_RR(namedb_type* db, const dname_type* dname,
 		domain_add_rrset(domain, rrset);
 	}
 
+	/* dnames in rdata are normalized, conform RFC 4035,
+	 * Section 6.2
+	 */
 	rdata_num = rdata_wireformat_to_rdata_atoms(
 		db->region, db->domains, type, rdatalen, packet, &rdatas);
 	if(rdata_num == -1) {
@@ -633,7 +659,7 @@ apply_ixfr(namedb_type* db, FILE *in, const off_t* startpos,
 	int* is_axfr, int* delete_mode, int* rr_count,
 	size_t child_count)
 {
-	uint32_t filelen, msglen, pkttype;
+	uint32_t filelen, msglen, pkttype, timestamp[2];
 	int qcount, ancount, counter;
 	buffer_type* packet;
 	region_type* region;
@@ -657,6 +683,12 @@ apply_ixfr(namedb_type* db, FILE *in, const off_t* startpos,
 		log_msg(LOG_ERR, "could not read type or wrong type");
 		return 0;
 	}
+	if(!diff_read_32(in, &timestamp[0]) ||
+	   !diff_read_32(in, &timestamp[1])) {
+		log_msg(LOG_ERR, "could not read timestamp");
+		return 0;
+	}
+
 	if(!diff_read_32(in, &filelen)) {
 		log_msg(LOG_ERR, "could not read len");
 		return 0;
@@ -1167,7 +1199,7 @@ store_ixfr_data(FILE *in, uint32_t len, struct diff_read_data* data, off_t* star
 		!diff_read_32(in, &new_serial) ||
 		!diff_read_16(in, &id) ||
 		!diff_read_32(in, &seq)) {
-		log_msg(LOG_INFO, "could not read ixfr store info: %s", strerror(errno));
+		log_msg(LOG_INFO, "could not read ixfr store info: file format error");
 		return 0;
 	}
 	len -= sizeof(uint32_t)*3 + sizeof(uint16_t) + strlen(zone_name);
@@ -1198,8 +1230,10 @@ read_process_part(namedb_type* db, FILE *in, uint32_t type,
 {
 	uint32_t len, len2;
 
-	if(!diff_read_32(in, &len)) return 1;
-
+	/* read length */
+	if(!diff_read_32(in, &len))
+		return 1;
+	/* read content */
 	if(type == DIFF_PART_IXFR) {
 		DEBUG(DEBUG_XFRD,2, (LOG_INFO, "part IXFR len %d", len));
 		if(!store_ixfr_data(in, len, data, startpos))
@@ -1213,8 +1247,10 @@ read_process_part(namedb_type* db, FILE *in, uint32_t type,
 		DEBUG(DEBUG_XFRD,1, (LOG_INFO, "unknown part %x len %d", type, len));
 		return 0;
 	}
+	/* read length */
 	if(!diff_read_32(in, &len2))
 		return 1; /* short read is OK */
+	/* verify length */
 	if(len != len2)
 		return 0; /* bad data is wrong */
 	return 1;
@@ -1263,7 +1299,7 @@ diff_read_file(namedb_type* db, nsd_options_t* opt, struct diff_log** log,
 {
 	const char* filename = opt->difffile;
 	FILE *df;
-	uint32_t type;
+	uint32_t type, timestamp[2], curr_timestamp[2];
 	struct diff_read_data* data = diff_read_data_create();
 	off_t startpos;
 
@@ -1274,11 +1310,54 @@ diff_read_file(namedb_type* db, nsd_options_t* opt, struct diff_log** log,
 		region_destroy(data->region);
 		return 1;
 	}
+
+	/* check timestamp */
+	curr_timestamp[0] = (uint32_t) db->diff_timestamp.tv_sec;
+	curr_timestamp[1] = (uint32_t) db->diff_timestamp.tv_usec;
+
+	if(!diff_read_32(df, &type)) {
+		DEBUG(DEBUG_XFRD,1, (LOG_INFO, "difffile %s is empty",
+			filename));
+		db->diff_skip = 0;
+		db->diff_pos = 0;
+	}
+	else if (!diff_read_32(df, &timestamp[0]) ||
+		 !diff_read_32(df, &timestamp[1])) {
+		log_msg(LOG_ERR, "difffile %s bad first part: no timestamp",
+			filename);
+		region_destroy(data->region);
+		return 0;
+	}
+	else if (curr_timestamp[0] != timestamp[0] ||
+		 curr_timestamp[1] != timestamp[1]) {
+		/* new timestamp, no skipping */
+		db->diff_timestamp.tv_sec = (time_t) timestamp[0];
+		db->diff_timestamp.tv_usec = (suseconds_t) timestamp[1];
+
+		if (db->diff_skip) {
+			DEBUG(DEBUG_XFRD,1, (LOG_INFO, "new timestamp on "
+				"difffile %s, restoring diff_skip and diff_pos "
+				"[old timestamp: %u.%u; new timestamp: %u.%u]",
+				filename, curr_timestamp[0], curr_timestamp[1],
+				timestamp[0], timestamp[1]));
+			db->diff_skip = 0;
+			db->diff_pos = 0;
+		}
+	}
+
+	/* Always seek, to diff_pos or to beginning of the file. */
+	if (fseeko(df, 0, SEEK_SET)==-1) {
+		log_msg(LOG_INFO, "could not fseeko file %s: %s.", filename,
+				strerror(errno));
+		region_destroy(data->region);
+		return 0;
+	}
 	if(db->diff_skip) {
 		DEBUG(DEBUG_XFRD,1, (LOG_INFO, "skip diff file"));
 		if(fseeko(df, db->diff_pos, SEEK_SET)==-1) {
-			log_msg(LOG_INFO, "could not fseeko file %s: %s. Reread from start.",
-				filename, strerror(errno));
+			log_msg(LOG_INFO, "could not fseeko file %s: %s. "
+					  "Reread from start.", filename,
+				strerror(errno));
 		}
 	}
 
@@ -1289,9 +1368,21 @@ diff_read_file(namedb_type* db, nsd_options_t* opt, struct diff_log** log,
 		return 0;
 	}
 
+	DEBUG(DEBUG_XFRD,1, (LOG_INFO, "start of diff file read at pos %u",
+		(uint32_t) db->diff_pos));
 	while(diff_read_32(df, &type))
 	{
 		DEBUG(DEBUG_XFRD,2, (LOG_INFO, "iter loop"));
+
+		/* read timestamp */
+		if(!diff_read_32(df, &timestamp[0]) ||
+			!diff_read_32(df, &timestamp[1])) {
+			log_msg(LOG_INFO, "could not read timestamp: %s.",
+				strerror(errno));
+			region_destroy(data->region);
+			return 0;
+		}
+
 		if(!read_process_part(db, df, type, opt, data, log,
 			child_count, &startpos))
 		{
@@ -1310,9 +1401,11 @@ diff_read_file(namedb_type* db, nsd_options_t* opt, struct diff_log** log,
 
 	if(find_smallest_offset(data, &db->diff_pos)) {
 		/* can skip to the first unused element */
+		DEBUG(DEBUG_XFRD,2, (LOG_INFO, "next time skip diff file"));
 		db->diff_skip = 1;
 	} else {
 		/* all processed, can skip to here next time */
+		DEBUG(DEBUG_XFRD,2, (LOG_INFO, "next time skip diff file"));
 		db->diff_skip = 1;
 		db->diff_pos = ftello(df);
 		if(db->diff_pos == -1) {
@@ -1375,7 +1468,7 @@ void diff_snip_garbage(namedb_type* db, nsd_options_t* opt)
 	if(db->diff_skip) {
 		DEBUG(DEBUG_XFRD,1, (LOG_INFO, "garbage collect skip diff file"));
 		if(fseeko(df, db->diff_pos, SEEK_SET)==-1) {
-			log_msg(LOG_INFO, "could not fseeko file %s: %s.", 
+			log_msg(LOG_INFO, "could not fseeko file %s: %s.",
 				filename, strerror(errno));
 			fclose(df);
 			return;
@@ -1386,13 +1479,12 @@ void diff_snip_garbage(namedb_type* db, nsd_options_t* opt)
 	if(diff_broken(df, &break_pos))
 	{
 		/* snip off at break_pos */
-		DEBUG(DEBUG_XFRD,1, (LOG_INFO, "snipping off trailing partial part of %s", 
+		DEBUG(DEBUG_XFRD,1, (LOG_INFO, "snipping off trailing partial part of %s",
 			filename));
 		if(ftruncate(fileno(df), break_pos) == -1)
-			log_msg(LOG_ERR, "ftruncate %s failed: %s", 
+			log_msg(LOG_ERR, "ftruncate %s failed: %s",
 				filename, strerror(errno));
 	}
 
 	fclose(df);
 }
-
