@@ -206,7 +206,7 @@ static rrtype_descriptor_type rrtype_descriptors[RRTYPE_DESCRIPTORS_LENGTH] = {
 	  { RDATA_ZF_CERTIFICATE_TYPE, RDATA_ZF_SHORT, RDATA_ZF_ALGORITHM,
 	    RDATA_ZF_BASE64 } },
 	/* 38 */
-	{ 38, NULL, T_UTYPE, 1, 1, { RDATA_WF_BINARY }, { RDATA_ZF_UNKNOWN } },
+	{ TYPE_A6, NULL, T_UTYPE, 1, 1, { RDATA_WF_BINARY }, { RDATA_ZF_UNKNOWN } },
 	/* 39 */
 	{ TYPE_DNAME, "DNAME", T_DNAME, 1, 1,
 	  { RDATA_WF_UNCOMPRESSED_DNAME }, { RDATA_ZF_DNAME } },
@@ -258,23 +258,23 @@ static rrtype_descriptor_type rrtype_descriptors[RRTYPE_DESCRIPTORS_LENGTH] = {
 	  { RDATA_WF_BYTE, RDATA_WF_BYTE, RDATA_WF_BINARY },
 	  { RDATA_ZF_BYTE, RDATA_ZF_BYTE, RDATA_ZF_HEX } },
 	/* 45 */
-	{ TYPE_IPSECKEY, "IPSECKEY", T_IPSECKEY, 4, 5, 
+	{ TYPE_IPSECKEY, "IPSECKEY", T_IPSECKEY, 4, 5,
 	  { RDATA_WF_BYTE, RDATA_WF_BYTE, RDATA_WF_BYTE, RDATA_WF_IPSECGATEWAY,
-	    RDATA_WF_BINARY }, 
+	    RDATA_WF_BINARY },
 	  { RDATA_ZF_BYTE, RDATA_ZF_BYTE, RDATA_ZF_BYTE, RDATA_ZF_IPSECGATEWAY,
 	    RDATA_ZF_BASE64 } },
 	/* 46 */
 	{ TYPE_RRSIG, "RRSIG", T_RRSIG, 9, 9,
 	  { RDATA_WF_SHORT, RDATA_WF_BYTE, RDATA_WF_BYTE, RDATA_WF_LONG,
 	    RDATA_WF_LONG, RDATA_WF_LONG, RDATA_WF_SHORT,
-	    RDATA_WF_UNCOMPRESSED_DNAME, RDATA_WF_BINARY },
+	    RDATA_WF_LITERAL_DNAME, RDATA_WF_BINARY },
 	  { RDATA_ZF_RRTYPE, RDATA_ZF_ALGORITHM, RDATA_ZF_BYTE, RDATA_ZF_PERIOD,
-	    RDATA_ZF_TIME, RDATA_ZF_TIME, RDATA_ZF_SHORT, RDATA_ZF_DNAME,
-	    RDATA_ZF_BASE64 } },
+	    RDATA_ZF_TIME, RDATA_ZF_TIME, RDATA_ZF_SHORT,
+		RDATA_ZF_LITERAL_DNAME, RDATA_ZF_BASE64 } },
 	/* 47 */
 	{ TYPE_NSEC, "NSEC", T_NSEC, 2, 2,
-	  { RDATA_WF_UNCOMPRESSED_DNAME, RDATA_WF_BINARY },
-	  { RDATA_ZF_DNAME, RDATA_ZF_NSEC } },
+	  { RDATA_WF_LITERAL_DNAME, RDATA_WF_BINARY },
+	  { RDATA_ZF_LITERAL_DNAME, RDATA_ZF_NSEC } },
 	/* 48 */
 	{ TYPE_DNSKEY, "DNSKEY", T_DNSKEY, 4, 4,
 	  { RDATA_WF_SHORT, RDATA_WF_BYTE, RDATA_WF_BYTE, RDATA_WF_BINARY },
@@ -283,9 +283,22 @@ static rrtype_descriptor_type rrtype_descriptors[RRTYPE_DESCRIPTORS_LENGTH] = {
 	/* 49 */
 	{ TYPE_DHCID, "DHCID", T_DHCID, 1, 1, { RDATA_WF_BINARY }, { RDATA_ZF_BASE64 } },
 	/* 50 */
-	{ 50, NULL, T_UTYPE, 1, 1, { RDATA_WF_BINARY }, { RDATA_ZF_UNKNOWN } },
+	{ TYPE_NSEC3, "NSEC3", T_NSEC3, 6, 6,
+	  { RDATA_WF_BYTE, /* hash type */
+	    RDATA_WF_BYTE, /* flags */
+	    RDATA_WF_SHORT, /* iterations */
+	    RDATA_WF_BINARYWITHLENGTH, /* salt */
+	    RDATA_WF_BINARYWITHLENGTH, /* next hashed name */
+	    RDATA_WF_BINARY /* type bitmap */ },
+	  { RDATA_ZF_BYTE, RDATA_ZF_BYTE, RDATA_ZF_SHORT, RDATA_ZF_HEX_LEN,
+	    RDATA_ZF_BASE32, RDATA_ZF_NSEC } },
 	/* 51 */
-	{ 51, NULL, T_UTYPE, 1, 1, { RDATA_WF_BINARY }, { RDATA_ZF_UNKNOWN } },
+	{ TYPE_NSEC3PARAM, "NSEC3PARAM", T_NSEC3PARAM, 4, 4,
+	  { RDATA_WF_BYTE, /* hash type */
+	    RDATA_WF_BYTE, /* flags */
+	    RDATA_WF_SHORT, /* iterations */
+	    RDATA_WF_BINARYWITHLENGTH /* salt */ },
+	  { RDATA_ZF_BYTE, RDATA_ZF_BYTE, RDATA_ZF_SHORT, RDATA_ZF_HEX_LEN } },
 	/* 52 */
 	{ 52, NULL, T_UTYPE, 1, 1, { RDATA_WF_BINARY }, { RDATA_ZF_UNKNOWN } },
 	/* 53 */
@@ -414,34 +427,13 @@ static rrtype_descriptor_type rrtype_descriptors[RRTYPE_DESCRIPTORS_LENGTH] = {
 	    RDATA_ZF_TEXT, RDATA_ZF_TEXT, RDATA_ZF_TEXT, RDATA_ZF_TEXT,
 	    RDATA_ZF_TEXT, RDATA_ZF_TEXT, RDATA_ZF_TEXT, RDATA_ZF_TEXT,
 	    RDATA_ZF_TEXT, RDATA_ZF_TEXT, RDATA_ZF_TEXT, RDATA_ZF_TEXT } },
-	/* larger type numbers */
-	{ TYPE_NSEC3, "NSEC3", T_NSEC3, 6, 6,
-	  { RDATA_WF_BYTE, /* hash type */
-	    RDATA_WF_BYTE, /* flags */
-	    RDATA_WF_SHORT, /* iterations */
-	    RDATA_WF_BINARYWITHLENGTH, /* salt */
-	    RDATA_WF_BINARYWITHLENGTH, /* next hashed name */
-	    RDATA_WF_BINARY /* type bitmap */ },
-	  { RDATA_ZF_BYTE, RDATA_ZF_BYTE, RDATA_ZF_SHORT, RDATA_ZF_HEX_LEN, 
-	    RDATA_ZF_BASE32, RDATA_ZF_NSEC } },
-	{ TYPE_NSEC3PARAM, "NSEC3PARAM", T_NSEC3PARAM, 4, 4,
-	  { RDATA_WF_BYTE, /* hash type */
-	    RDATA_WF_BYTE, /* flags */
-	    RDATA_WF_SHORT, /* iterations */
-	    RDATA_WF_BINARYWITHLENGTH /* salt */ },
-	  { RDATA_ZF_BYTE, RDATA_ZF_BYTE, RDATA_ZF_SHORT, RDATA_ZF_HEX_LEN } },
 };
 
 rrtype_descriptor_type *
 rrtype_descriptor_by_type(uint16_t type)
 {
-	int i;
-
-	if (type < RRTYPE_DESCRIPTORS_IDX_LEN)
+	if (type < RRTYPE_DESCRIPTORS_LENGTH)
 		return &rrtype_descriptors[type];
-	for (i=RRTYPE_DESCRIPTORS_IDX_LEN; i<RRTYPE_DESCRIPTORS_LENGTH; ++i)
-		if (rrtype_descriptors[i].type == type)
-			return &rrtype_descriptors[i];
 	return &rrtype_descriptors[0];
 }
 
@@ -494,20 +486,20 @@ rrtype_from_string(const char *name)
 
 	if (strlen(name) < 5)
 		return 0;
-	
+
 	if (strncasecmp(name, "TYPE", 4) != 0)
 		return 0;
 
-	if (!isdigit(name[4]))
+	if (!isdigit((int)name[4]))
 		return 0;
-	
+
 	/* The rest from the string must be a number.  */
 	rrtype = strtol(name + 4, &end, 10);
 	if (*end != '\0')
 		return 0;
 	if (rrtype < 0 || rrtype > 65535L)
 		return 0;
-	
+
         return (uint16_t) rrtype;
 }
 
@@ -539,20 +531,19 @@ rrclass_from_string(const char *name)
 
 	if (strlen(name) < 6)
 		return 0;
-	
+
 	if (strncasecmp(name, "CLASS", 5) != 0)
 		return 0;
 
-	if (!isdigit(name[5]))
+	if (!isdigit((int)name[5]))
 		return 0;
-	
+
 	/* The rest from the string must be a number.  */
 	rrclass = strtol(name + 5, &end, 10);
 	if (*end != '\0')
 		return 0;
 	if (rrclass < 0 || rrclass > 65535L)
 		return 0;
-	
-        return (uint16_t) rrclass;
-	
+
+	return (uint16_t) rrclass;
 }
