@@ -114,6 +114,8 @@ typedef enum nsd_rc nsd_rc_type;
 #define TYPE_KX		36	/* RFC2230 Key Exchange Delegation Record */
 #define TYPE_CERT	37	/* RFC2538 */
 
+#define TYPE_A6		38	/* RFC2874 */
+
 #define TYPE_DNAME	39	/* RFC2672 */
 
 #define TYPE_OPT	41	/* Pseudo OPT record... */
@@ -126,13 +128,11 @@ typedef enum nsd_rc nsd_rc_type;
 #define TYPE_NSEC	47	/* RFC 4033, 4034, and 4035 */
 #define TYPE_DNSKEY	48	/* RFC 4033, 4034, and 4035 */
 #define TYPE_DHCID	49	/* RFC4701 DHCP information */
+#define TYPE_NSEC3	50	/* NSEC3, secure denial, prevents zonewalking */
+#define TYPE_NSEC3PARAM 51	/* NSEC3PARAM at zone apex nsec3 parameters */
 
 #define TYPE_SPF        99      /* RFC 4408 */
 
-/* high type range RRTYPES */
-#define TYPE_NSEC3	65324	/* NSEC3, dns secure denial, prevents zonewalking */
-#define TYPE_NSEC3PARAM 65325	/* NSEC3PARAM at zone apex nsec3 parameters */
-
 #define TYPE_TSIG	250
 #define TYPE_IXFR	251
 #define TYPE_AXFR	252
@@ -160,6 +160,7 @@ enum rdata_wireformat
 {
 	RDATA_WF_COMPRESSED_DNAME,   /* Possibly compressed domain name.  */
 	RDATA_WF_UNCOMPRESSED_DNAME, /* Uncompressed domain name.  */
+	RDATA_WF_LITERAL_DNAME,      /* Literal (not downcased) dname.  */
 	RDATA_WF_BYTE,		     /* 8-bit integer.  */
 	RDATA_WF_SHORT,		     /* 16-bit integer.  */
 	RDATA_WF_LONG,		     /* 32-bit integer.  */
@@ -169,7 +170,7 @@ enum rdata_wireformat
 	RDATA_WF_BINARY, 	     /* Binary data (unknown length).  */
 	RDATA_WF_BINARYWITHLENGTH,   /* Binary data preceded by 1 byte length */
 	RDATA_WF_APL,		     /* APL data.  */
-	RDATA_WF_IPSECGATEWAY,	     /* IPSECKEY gateway ip4, ip6 or dname. */
+	RDATA_WF_IPSECGATEWAY	     /* IPSECKEY gateway ip4, ip6 or dname. */
 };
 typedef enum rdata_wireformat rdata_wireformat_type;
 
@@ -179,6 +180,7 @@ typedef enum rdata_wireformat rdata_wireformat_type;
 enum rdata_zoneformat
 {
 	RDATA_ZF_DNAME,		/* Domain name.  */
+	RDATA_ZF_LITERAL_DNAME,	/* DNS name (not lowercased domain name).  */
 	RDATA_ZF_TEXT,		/* Text string.  */
 	RDATA_ZF_BYTE,		/* 8-bit integer.  */
 	RDATA_ZF_SHORT,		/* 16-bit integer.  */
@@ -221,11 +223,9 @@ typedef struct rrtype_descriptor rrtype_descriptor_type;
  * Indexed by type.  The special type "0" can be used to get a
  * descriptor for unknown types (with one binary rdata).
  *
- * spf + 1 + the number of high-type-range RRTYPES
+ * spf + 1
  */
-#define RRTYPE_DESCRIPTORS_LENGTH  (TYPE_SPF + 1 + 2)
-/* below this value the types are consecutive numbered */
-#define RRTYPE_DESCRIPTORS_IDX_LEN (TYPE_SPF + 1)
+#define RRTYPE_DESCRIPTORS_LENGTH  (TYPE_SPF + 1)
 rrtype_descriptor_type *rrtype_descriptor_by_name(const char *name);
 rrtype_descriptor_type *rrtype_descriptor_by_type(uint16_t type);
 
