@@ -14,6 +14,7 @@
 #include <netinet/in.h>
 #include <arpa/inet.h>
 
+#include <limits.h>
 #include <assert.h>
 #include <errno.h>
 #include <netdb.h>
@@ -109,6 +110,12 @@ int check_matching_address_family(struct addrinfo *a, struct addrinfo *b);
  */
 struct addrinfo *find_by_address_family(struct addrinfo *addrs, int family);
 
+/*
+ * Assigns pointers to hostname and port and wipes out the optional delimiter.
+ */
+void get_hostname_port_frm_str(char* arg, const char** hostname,
+	const char** port);
+
 /*
  * Log an error message and exit.
  */
@@ -147,13 +154,14 @@ usage (void)
 	fprintf(stderr,
 		"Usage: nsd-xfer [OPTION]... -z zone -f file server...\n"
 		"NSD AXFR client.\n\nSupported options:\n"
-		"  -4           Only use IPv4 connections.\n"
-		"  -6           Only use IPv6 connections.\n"
-		"  -a src       Local hostname/IP for the connection.\n"
-		"  -f file      Output zone file name.\n"
-		"  -p port      The port to connect to.\n"
-		"  -s serial    The current zone serial.\n"
-		"  -T tsiginfo  The TSIG key file name.  The file is removed "
+		"  -4            Only use IPv4 connections.\n"
+		"  -6            Only use IPv6 connections.\n"
+		"  -a src[@port] Local hostname/ip-address for the \
+connection, including optional source port.\n"
+		"  -f file       Output zone file name.\n"
+		"  -p port       The port to connect to.\n"
+		"  -s serial     The current zone serial.\n"
+		"  -T tsiginfo   The TSIG key file name.  The file is removed "
 		"after reading the\n               key.\n"
 		"  -v           Verbose output.\n");
 	fprintf(stderr,
@@ -194,23 +202,27 @@ read_line(FILE *in, char *line, size_t size)
 	}
 }
 
+#ifdef TSIG
 static tsig_key_type *
-read_tsig_key_data(region_type *region, FILE *in, 
-	int ATTR_UNUSED(default_family))
+read_tsig_key_data(region_type *region, FILE *in,
+	int ATTR_UNUSED(default_family), tsig_algorithm_type** tsig_algo)
 {
 	char line[4000];
 	tsig_key_type *key = (tsig_key_type *) region_alloc(
 		region, sizeof(tsig_key_type));
 	int size;
+	uint8_t algo = 0;
 	uint8_t data[4000];
 
+	/* server address */
 	if (!read_line(in, line, sizeof(line))) {
 		error("failed to read TSIG key server address: '%s'",
 		      strerror(errno));
 		return NULL;
 	}
-	/* server name unused */
+	/* server address unused */
 
+	/* tsig keyname */
 	if (!read_line(in, line, sizeof(line))) {
 		error("failed to read TSIG key name: '%s'", strerror(errno));
 		return NULL;
@@ -222,11 +234,19 @@ read_tsig_key_data(region_type *region, FILE *in,
 		return NULL;
 	}
 
+	/* tsig algorithm */
 	if (!read_line(in, line, sizeof(line))) {
-		error("failed to read TSIG key type: '%s'", strerror(errno));
+		error("failed to read TSIG key algorithm: '%s'", strerror(errno));
+		return NULL;
+	}
+	algo = (uint8_t) atoi((const char*) line);
+	*tsig_algo = tsig_get_algorithm_by_id(algo);
+	if (*tsig_algo == NULL) {
+		error("failed to parse TSIG key algorithm %i: '%s'\n", algo, strerror(errno));
 		return NULL;
 	}
 
+	/* tsig secret */
 	if (!read_line(in, line, sizeof(line))) {
 		error("failed to read TSIG key data: '%s'\n", strerror(errno));
 		return NULL;
@@ -250,7 +270,7 @@ read_tsig_key_data(region_type *region, FILE *in,
 static tsig_key_type *
 read_tsig_key(region_type *region,
 	      const char *tsiginfo_filename,
-	      int default_family)
+	      int default_family, tsig_algorithm_type** algo)
 {
 	FILE *in;
 	tsig_key_type *key;
@@ -263,7 +283,7 @@ read_tsig_key(region_type *region,
 		return NULL;
 	}
 
-	key = read_tsig_key_data(region, in, default_family);
+	key = read_tsig_key_data(region, in, default_family, algo);
 
 	fclose(in);
 
@@ -275,6 +295,7 @@ read_tsig_key(region_type *region,
 
 	return key;
 }
+#endif /* TSIG */
 
 /*
  * Read SIZE bytes from the socket into BUF.  Keep reading unless an
@@ -761,6 +782,7 @@ main(int argc, char *argv[])
 	const char *local_hostname = NULL;
 	struct addrinfo *local_address, *local_addresses = NULL;
 	const char *port = TCP_PORT;
+	const char *local_port = NULL;
 	int default_family = DEFAULT_AI_FAMILY;
 	struct sigaction mysigaction;
 	FILE *zone_file;
@@ -812,7 +834,8 @@ main(int argc, char *argv[])
 #endif /* !INET6 */
 			break;
 		case 'a':
-			local_hostname = optarg;
+			get_hostname_port_frm_str((char *) optarg,
+				&local_hostname, &local_port);
 			break;
 		case 'f':
 			zone_filename = optarg;
@@ -824,18 +847,26 @@ main(int argc, char *argv[])
 			port = optarg;
 			break;
 		case 's': {
-			const char *t;
-			state.first_transfer = 0;
-			state.last_serial = strtottl(optarg, &t);
-			if (*t != '\0') {
+			long long v;
+			char *t;
+ 			state.first_transfer = 0;
+			v = strtoll(optarg, &t, 10);
+			if (optarg[0] == '\0' || *t != '\0' || v < 0 || v > UINT_MAX
+				|| (errno == ERANGE && (v == LLONG_MIN || v == LLONG_MAX)))
+			{
 				error("bad serial '%s'", optarg);
 				exit(XFER_FAIL);
 			}
+			state.last_serial = (uint32_t) v;
 			break;
 		}
 		case 'T':
+#ifdef TSIG
 			tsig_key_filename = optarg;
 			break;
+#else
+			log_msg(LOG_ERR, "option -T given but TSIG not enabled");
+#endif /* TSIG */
 		case 'v':
 			++state.verbose;
 			break;
@@ -856,18 +887,13 @@ main(int argc, char *argv[])
 	if (argc == 0 || !zone_filename || !state.zone)
 		usage();
 
+#ifdef TSIG
 	if (tsig_key_filename) {
-		tsig_algorithm_type *md5
-			= tsig_get_algorithm_by_name("hmac-md5");
-		if (!md5) {
-			error("cannot initialize hmac-md5: TSIG support not"
-			      " enabled");
-			exit(XFER_FAIL);
-		}
-
+		tsig_algorithm_type *tsig_algo = NULL;
 		tsig_key = read_tsig_key(
-			region, tsig_key_filename, default_family);
+			region, tsig_key_filename, default_family, &tsig_algo);
 		if (!tsig_key) {
+			error("cannot initialize TSIG: error in tsiginfo file");
 			exit(XFER_FAIL);
 		}
 
@@ -876,8 +902,9 @@ main(int argc, char *argv[])
 		state.tsig = (tsig_record_type *) region_alloc(
 			region, sizeof(tsig_record_type));
 		tsig_create_record(state.tsig, region);
-		tsig_init_record(state.tsig, md5, tsig_key);
+		tsig_init_record(state.tsig, tsig_algo, tsig_key);
 	}
+#endif /* TSIG */
 
 	mysigaction.sa_handler = to_alarm;
 	sigfillset(&mysigaction.sa_mask);
@@ -892,23 +919,21 @@ main(int argc, char *argv[])
 	hints.ai_protocol = IPPROTO_TCP;
 
 	if (local_hostname) {
-		int rc = getaddrinfo(local_hostname, NULL,
+		int rc = getaddrinfo(local_hostname, local_port,
 				     &hints, &local_addresses);
 		if (rc) {
 			error("local hostname '%s' not found: %s",
-			      local_hostname,
-			      gai_strerror(rc));
+				local_hostname, gai_strerror(rc));
 		}
 	}
 
-	for (; *argv; ++argv) {
+	for (/*empty*/; *argv; ++argv) {
 		/* Try each server separately until one succeeds.  */
 		int rc;
 
 		rc = getaddrinfo(*argv, port, &hints, &res0);
 		if (rc) {
-			warning("skipping bad address %s: %s\n",
-				*argv,
+			warning("skipping bad address %s: %s\n", *argv,
 				gai_strerror(rc));
 			continue;
 		}
@@ -932,7 +957,7 @@ main(int argc, char *argv[])
 			 * address.
 			 */
 			local_address = find_by_address_family(local_addresses,
-							       res->ai_family);
+				res->ai_family);
 			if (local_addresses && !local_address) {
 				/* Continue with next remote address.  */
 				continue;
@@ -947,14 +972,12 @@ main(int argc, char *argv[])
 			}
 
 			/* Bind socket to local address, if required.  */
-			if (local_address
-			    && bind(state.s,
-				    local_address->ai_addr,
-				    local_address->ai_addrlen) < 0)
+			if (local_address && bind(state.s,
+				local_address->ai_addr,
+				local_address->ai_addrlen) < 0)
 			{
 				warning("cannot bind to %s: %s\n",
-					local_hostname,
-					strerror(errno));
+					local_hostname, strerror(errno));
 			}
 
 			if (connect(state.s, res->ai_addr, res->ai_addrlen) < 0)
@@ -1014,6 +1037,21 @@ main(int argc, char *argv[])
 	exit(XFER_FAIL);
 }
 
+void
+get_hostname_port_frm_str(char* arg, const char** hostname,
+	const char** port)
+{
+	/* parse -a src[@port] option */
+	char* delim = strchr(arg, '@');
+
+	if (delim) {
+		*delim = '\0';
+		*port = delim+1;
+	}
+	*hostname = arg;
+}
+
+
 int
 check_matching_address_family(struct addrinfo *a0, struct addrinfo *b0)
 {
