@@ -46,6 +46,10 @@ static char hostname[MAXHOSTNAMELEN];
 
 static void error(const char *format, ...) ATTR_FORMAT(printf, 1, 2);
 
+/*
+ * Print the help text.
+ *
+ */
 static void
 usage (void)
 {
@@ -86,10 +90,14 @@ usage (void)
 		"  -V level        Specify verbosity level.\n"
 		"  -v              Print version information.\n"
 		);
-	fprintf(stderr, "Version %s. Report bugs to <%s>.\n", 
+	fprintf(stderr, "Version %s. Report bugs to <%s>.\n",
 		PACKAGE_VERSION, PACKAGE_BUGREPORT);
 }
 
+/*
+ * Print the version exit.
+ *
+ */
 static void
 version(void)
 {
@@ -102,6 +110,10 @@ version(void)
 	exit(0);
 }
 
+/*
+ * Something went wrong, give error messages and exit.
+ *
+ */
 static void
 error(const char *format, ...)
 {
@@ -112,8 +124,13 @@ error(const char *format, ...)
 	exit(1);
 }
 
-pid_t 
-readpid (const char *file)
+
+/*
+ * Fetch the nsd parent process id from the nsd pidfile
+ *
+ */
+pid_t
+readpid(const char *file)
 {
 	int fd;
 	pid_t pid;
@@ -146,8 +163,12 @@ readpid (const char *file)
 	return pid;
 }
 
-int 
-writepid (struct nsd *nsd)
+/*
+ * Store the nsd parent process id in the nsd pidfile
+ *
+ */
+int
+writepid(struct nsd *nsd)
 {
 	FILE * fd;
 	char pidbuf[32];
@@ -155,10 +176,14 @@ writepid (struct nsd *nsd)
 	snprintf(pidbuf, sizeof(pidbuf), "%lu\n", (unsigned long) nsd->pid);
 
 	if ((fd = fopen(nsd->pidfile, "w")) ==  NULL ) {
+		log_msg(LOG_ERR, "cannot open pidfile %s: %s",
+			nsd->pidfile, strerror(errno));
 		return -1;
 	}
 
 	if (!write_data(fd, pidbuf, strlen(pidbuf))) {
+		log_msg(LOG_ERR, "cannot write pidfile %s: %s",
+			nsd->pidfile, strerror(errno));
 		fclose(fd);
 		return -1;
 	}
@@ -173,13 +198,16 @@ writepid (struct nsd *nsd)
 
 	return 0;
 }
-	
 
-void 
-sig_handler (int sig)
+/*
+ * Incoming signals, set appropriate actions.
+ *
+ */
+void
+sig_handler(int sig)
 {
 	/* To avoid race cond. We really don't want to use log_msg() in this handler */
-	
+
 	/* Are we a child server? */
 	if (nsd.server_kind != NSD_SERVER_MAIN) {
 		switch (sig) {
@@ -203,6 +231,7 @@ sig_handler (int sig)
 		return;
 	}
 
+	/* We are the main process */
 	switch (sig) {
 	case SIGCHLD:
 		nsd.signal_hint_child = 1;
@@ -240,7 +269,7 @@ sig_handler (int sig)
  *
  */
 #ifdef BIND8_STATS
-void 
+void
 bind8_stats (struct nsd *nsd)
 {
 	char buf[MAXSYSLOGMSGLEN];
@@ -301,24 +330,23 @@ bind8_stats (struct nsd *nsd)
 extern char *optarg;
 extern int optind;
 
-int 
-main (int argc, char *argv[])
+int
+main(int argc, char *argv[])
 {
 	/* Scratch variables... */
 	int c;
 	pid_t	oldpid;
 	size_t i;
 	struct sigaction action;
-	
+
 	/* For initialising the address info structures */
 	struct addrinfo hints[MAX_INTERFACES];
 	const char *nodes[MAX_INTERFACES];
 	const char *udp_port = 0;
 	const char *tcp_port = 0;
 
-	const char *log_filename = NULL;
 	const char *configfile = CONFIGFILE;
-	
+
 	log_init("nsd");
 
 	/* Initialize the server handler... */
@@ -327,7 +355,7 @@ main (int argc, char *argv[])
 	nsd.dbfile	= 0;
 	nsd.pidfile	= 0;
 	nsd.server_kind = NSD_SERVER_MAIN;
-	
+
 	for (i = 0; i < MAX_INTERFACES; i++) {
 		memset(&hints[i], 0, sizeof(hints[i]));
 		hints[i].ai_family = DEFAULT_AI_FAMILY;
@@ -346,16 +374,17 @@ main (int argc, char *argv[])
 	nsd.maximum_tcp_count = 0;
 	nsd.current_tcp_count = 0;
 	nsd.grab_ip6_optional = 0;
-	
+	nsd.file_rotation_ok = 0;
+
 	/* EDNS0 */
 	edns_init_data(&nsd.edns_ipv4, EDNS_MAX_MESSAGE_LEN);
 #if defined(INET6)
-# if defined(IPV6_USE_MIN_MTU)
+#if defined(IPV6_USE_MIN_MTU)
 	edns_init_data(&nsd.edns_ipv6, EDNS_MAX_MESSAGE_LEN);
-# else /* !defined(IPV6_USE_MIN_MTU) */
+#else /* !defined(IPV6_USE_MIN_MTU) */
 	edns_init_data(&nsd.edns_ipv6, IPV6_MIN_MTU);
-# endif
-#endif
+#endif /* defined(IPV6_USE_MIN_MTU) */
+#endif /* defined(INET6) */
 
 	/* Set up our default identity to gethostname(2) */
 	if (gethostname(hostname, MAXHOSTNAMELEN) == 0) {
@@ -370,7 +399,7 @@ main (int argc, char *argv[])
 
 	/* Parse the command line... */
 	while ((c = getopt(argc, argv, "46a:c:df:hi:I:l:N:n:P:p:s:u:t:X:V:v"
-#ifndef NDEBUG	
+#ifndef NDEBUG /* <mattthijs> only when configured with --enable-checking */
 		"F:L:"
 #endif /* NDEBUG */
 		)) != -1) {
@@ -387,7 +416,7 @@ main (int argc, char *argv[])
 			}
 #else /* !INET6 */
 			error("IPv6 support not enabled.");
-#endif /* !INET6 */
+#endif /* INET6 */
 			break;
 		case 'a':
 			if (nsd.ifs < MAX_INTERFACES) {
@@ -429,7 +458,7 @@ main (int argc, char *argv[])
 #endif /* NSID */
                        break;
 		case 'l':
-			log_filename = optarg;
+			nsd.log_filename = optarg;
 			break;
 		case 'N':
 			i = atoi(optarg);
@@ -462,14 +491,14 @@ main (int argc, char *argv[])
 			nsd.st.period = atoi(optarg);
 #else /* !BIND8_STATS */
 			error("BIND 8 statistics not enabled.");
-#endif /* !BIND8_STATS */
+#endif /* BIND8_STATS */
 			break;
 		case 't':
 #ifdef HAVE_CHROOT
 			nsd.chrootdir = optarg;
 #else /* !HAVE_CHROOT */
 			error("chroot not supported on this platform.");
-#endif /* !HAVE_CHROOT */
+#endif /* HAVE_CHROOT */
 			break;
 		case 'u':
 			nsd.username = optarg;
@@ -497,6 +526,7 @@ main (int argc, char *argv[])
 	argc -= optind;
 	argv += optind;
 
+	/* Commandline parse error */
 	if (argc != 0) {
 		usage();
 		exit(1);
@@ -507,11 +537,11 @@ main (int argc, char *argv[])
 		      (unsigned) strlen(nsd.identity));
 	}
 
-        /* Read options */
-        nsd.options = nsd_options_create(region_create(xalloc, free));
-        if(!parse_options_file(nsd.options, configfile)) {
-                error("nsd: could not read config: %s\n", configfile);
-        }
+	/* Read options */
+	nsd.options = nsd_options_create(region_create(xalloc, free));
+	if(!parse_options_file(nsd.options, configfile)) {
+		error("could not read config: %s\n", configfile);
+	}
 	if(nsd.options->ip4_only) {
 		for (i = 0; i < MAX_INTERFACES; ++i) {
 			hints[i].ai_family = AF_INET;
@@ -523,7 +553,7 @@ main (int argc, char *argv[])
 			hints[i].ai_family = AF_INET6;
 		}
 	}
-#endif /* !INET6 */
+#endif /* INET6 */
 	if(nsd.options->ip_addresses)
 	{
 		ip_address_option_t* ip = nsd.options->ip_addresses;
@@ -532,7 +562,8 @@ main (int argc, char *argv[])
 				nodes[nsd.ifs] = ip->address;
 				++nsd.ifs;
 			} else {
-				error("too many interfaces ('-a' + 'ip-address:') specified.");
+				error("too many interfaces ('-a' + "
+				      "'ip-address:') specified.");
 				break;
 			}
 			ip = ip->next;
@@ -547,20 +578,25 @@ main (int argc, char *argv[])
 	if(nsd.options->debug_mode) nsd.debug=1;
 	if(!nsd.dbfile)
 	{
-		if(nsd.options->database) nsd.dbfile = nsd.options->database;
-		else nsd.dbfile = DBFILE;
+		if(nsd.options->database)
+			nsd.dbfile = nsd.options->database;
+		else
+			nsd.dbfile = DBFILE;
 	}
 	if(!nsd.pidfile)
 	{
-		if(nsd.options->pidfile) nsd.pidfile = nsd.options->pidfile;
-		else nsd.pidfile = PIDFILE;
+		if(nsd.options->pidfile)
+			nsd.pidfile = nsd.options->pidfile;
+		else
+			nsd.pidfile = PIDFILE;
 	}
-	if(nsd.identity == hostname || nsd.identity == IDENTITY)
+	if(strcmp(nsd.identity, hostname)==0 || strcmp(nsd.identity,IDENTITY)==0)
 	{
-		if(nsd.options->identity) nsd.identity = nsd.options->identity;
+		if(nsd.options->identity)
+			nsd.identity = nsd.options->identity;
 	}
-	if (nsd.options->logfile && !log_filename) {
-		log_filename = nsd.options->logfile;
+	if (nsd.options->logfile && !nsd.log_filename) {
+		nsd.log_filename = nsd.options->logfile;
 	}
 	if(nsd.child_count == 0) {
 		nsd.child_count = nsd.options->server_count;
@@ -582,27 +618,29 @@ main (int argc, char *argv[])
 	if(nsd.st.period == 0) {
 		nsd.st.period = nsd.options->statistics;
 	}
-#endif /* !BIND8_STATS */
+#endif /* BIND8_STATS */
 #ifdef HAVE_CHROOT
 	if(nsd.chrootdir == 0) nsd.chrootdir = nsd.options->chroot;
-#endif
+#endif /* HAVE_CHROOT */
 	if(nsd.username == 0) {
 		if(nsd.options->username) nsd.username = nsd.options->username;
 		else nsd.username = USER;
 	}
 	if(nsd.options->zonesdir && nsd.options->zonesdir[0]) {
 		if(chdir(nsd.options->zonesdir)) {
-			error("cannot chdir to '%s': %s", 
+			error("cannot chdir to '%s': %s",
 				nsd.options->zonesdir, strerror(errno));
 		}
+		DEBUG(DEBUG_IPC,1, (LOG_INFO, "changed directory to %s",
+			nsd.options->zonesdir));
 	}
-	/* get it from the config file */
+
 #ifdef NSID
 	edns_init_nsid(&nsd.edns_ipv4, nsd.nsid_len);
 #if defined(INET6)
 	edns_init_nsid(&nsd.edns_ipv6, nsd.nsid_len);
-#endif
-#endif
+#endif /* defined(INET6) */
+#endif /* NSID */
 	/* Number of child servers to fork.  */
 	nsd.children = (struct nsd_child *) region_alloc(
 		nsd.region, nsd.child_count * sizeof(struct nsd_child));
@@ -616,12 +654,12 @@ main (int argc, char *argv[])
 		nsd.children[i].need_to_send_QUIT = 0;
 		nsd.children[i].need_to_exit = 0;
 		nsd.children[i].has_exited = 0;
-		nsd.children[i].dirty_zones = stack_create(nsd.region, 
+		nsd.children[i].dirty_zones = stack_create(nsd.region,
 			nsd_options_num_zones(nsd.options));
 	}
 
 	nsd.this_child = NULL;
-	
+
 	/* We need at least one active interface */
 	if (nsd.ifs == 0) {
 		nsd.ifs = 1;
@@ -640,14 +678,14 @@ main (int argc, char *argv[])
 		 */
 #ifdef INET6
 		if (hints[0].ai_family == AF_UNSPEC) {
-# ifdef IPV6_V6ONLY
+#ifdef IPV6_V6ONLY
 			hints[0].ai_family = AF_INET6;
 			hints[1].ai_family = AF_INET;
 			nsd.ifs = 2;
 			nsd.grab_ip6_optional = 1;
-# else /* !IPV6_V6ONLY */
+#else /* !IPV6_V6ONLY */
 			hints[0].ai_family = AF_INET6;
-# endif	/* !IPV6_V6ONLY */
+#endif	/* IPV6_V6ONLY */
 		}
 #endif /* INET6 */
 	}
@@ -659,12 +697,12 @@ main (int argc, char *argv[])
 		/* We don't perform name-lookups */
 		if (nodes[i] != NULL)
 			hints[i].ai_flags |= AI_NUMERICHOST;
-		
+
 		hints[i].ai_socktype = SOCK_DGRAM;
 		if ((r=getaddrinfo(nodes[i], udp_port, &hints[i], &nsd.udp[i].addr)) != 0) {
 #ifdef INET6
 			if(nsd.grab_ip6_optional && hints[0].ai_family == AF_INET6) {
-				log_msg(LOG_WARNING, "No IPv6, fallback to IPv4. getaddrinfo: %s", 
+				log_msg(LOG_WARNING, "No IPv6, fallback to IPv4. getaddrinfo: %s",
 				r==EAI_SYSTEM?strerror(errno):gai_strerror(r));
 				continue;
 			}
@@ -674,7 +712,7 @@ main (int argc, char *argv[])
 				gai_strerror(r),
 				r==EAI_SYSTEM?strerror(errno):"");
 		}
-		
+
 		hints[i].ai_socktype = SOCK_STREAM;
 		if ((r=getaddrinfo(nodes[i], tcp_port, &hints[i], &nsd.tcp[i].addr)) != 0) {
 			error("cannot parse address '%s': getaddrinfo: %s %s",
@@ -689,11 +727,11 @@ main (int argc, char *argv[])
 	nsd.uid = getuid();
 	if (*nsd.username) {
 		struct passwd *pwd;
-		if (isdigit(*nsd.username)) {
+		if (isdigit((int)*nsd.username)) {
 			char *t;
 			nsd.uid = strtol(nsd.username, &t, 10);
 			if (*t != 0) {
-				if (*t != '.' || !isdigit(*++t)) {
+				if (*t != '.' || !isdigit((int)*++t)) {
 					error("-u user or -u uid or -u uid.gid");
 				}
 				nsd.gid = strtol(t, &t, 10);
@@ -724,16 +762,26 @@ main (int argc, char *argv[])
 	key_options_tsig_add(nsd.options);
 #endif /* TSIG */
 
-	/* Set up the logging... */
-	log_open(LOG_PID, FACILITY, log_filename);
-	if (!log_filename) {
+	/* Set up the logging */
+	log_open(LOG_PID, FACILITY, nsd.log_filename);
+	if (!nsd.log_filename)
 		log_set_log_function(log_syslog);
-	}
-	
+	else if (nsd.uid && nsd.gid)
+		(void) chown(nsd.log_filename, nsd.uid, nsd.gid);
+
 	/* Relativize the pathnames for chroot... */
 	if (nsd.chrootdir) {
 		int l = strlen(nsd.chrootdir);
 
+		/* existing chrootdir: append trailing slash for strncmp checking */
+		if (l>0 && strncmp(nsd.chrootdir + (l-1), "/", 1) != 0) {
+			char *chroot_slash = region_alloc(nsd.region, sizeof(char)*(l+2));
+			memcpy(chroot_slash, nsd.chrootdir, sizeof(char)*(l+1));
+			strncat(chroot_slash, "/", 1);
+			nsd.chrootdir = chroot_slash;
+			++l;
+		}
+
 		if (strncmp(nsd.chrootdir, nsd.pidfile, l) != 0) {
 			log_msg(LOG_ERR, "%s is not relative to %s: will not chroot",
 				nsd.pidfile, nsd.chrootdir);
@@ -774,16 +822,17 @@ main (int argc, char *argv[])
 	/* Unless we're debugging, fork... */
 	if (!nsd.debug) {
 		int fd;
-		
+
 		/* Take off... */
 		switch ((nsd.pid = fork())) {
 		case 0:
+			/* Child */
 			break;
 		case -1:
-			log_msg(LOG_ERR, "fork failed: %s", strerror(errno));
-			unlink(nsd.pidfile);
+			log_msg(LOG_ERR, "fork() failed: %s", strerror(errno));
 			exit(1);
 		default:
+			/* Parent is done */
 			exit(0);
 		}
 
@@ -816,16 +865,9 @@ main (int argc, char *argv[])
 	action.sa_handler = SIG_IGN;
 	sigaction(SIGPIPE, &action, NULL);
 
-
 	/* Get our process id */
 	nsd.pid = getpid();
 
-	/* Overwrite pid... */
-	if (writepid(&nsd) == -1) {
-		log_msg(LOG_ERR, "cannot overwrite the pidfile %s: %s",
-			nsd.pidfile, strerror(errno));
-	}
-
 	/* Initialize... */
 	nsd.mode = NSD_RUN;
 	nsd.signal_hint_child = 0;
@@ -838,11 +880,12 @@ main (int argc, char *argv[])
 
 	/* Run the server... */
 	if (server_init(&nsd) != 0) {
-		unlink(nsd.pidfile);
+		log_msg(LOG_ERR, "server initialization failed, nsd could "
+						 "not be started");
 		exit(1);
 	}
 
-	log_msg(LOG_NOTICE, "nsd started (%s), pid %d", PACKAGE_STRING, 
+	log_msg(LOG_NOTICE, "nsd started (%s), pid %d", PACKAGE_STRING,
 		(int) nsd.pid);
 
 	if (nsd.server_kind == NSD_SERVER_MAIN) {
