@@ -53,9 +53,9 @@ static void answer_authoritative(struct nsd	  *nsd,
 				 domain_type      *closest_encloser,
 				 const dname_type *qname);
 
-static void answer_lookup_zone(struct nsd *nsd, struct query *q, 
-			       answer_type *answer, uint32_t domain_number, 
-			       int exact, domain_type *closest_match, 
+static void answer_lookup_zone(struct nsd *nsd, struct query *q,
+			       answer_type *answer, uint32_t domain_number,
+			       int exact, domain_type *closest_match,
 			       domain_type *closest_encloser,
 			       const dname_type *qname);
 
@@ -70,7 +70,7 @@ query_put_dname_offset(struct query *q, domain_type *domain, uint16_t offset)
 		return;
 	if (q->compressed_dname_count >= MAX_COMPRESSED_DNAMES)
 		return;
-	
+
 	q->compressed_dname_offsets[domain->number] = offset;
 	q->compressed_dnames[q->compressed_dname_count] = domain;
 	++q->compressed_dname_count;
@@ -92,7 +92,7 @@ void
 query_clear_compression_tables(struct query *q)
 {
 	uint16_t i;
-	
+
 	for (i = 0; i < q->compressed_dname_count; ++i) {
 		assert(q->compressed_dnames);
 		q->compressed_dname_offsets[q->compressed_dnames[i]->number] = 0;
@@ -124,12 +124,12 @@ query_error (struct query *q, nsd_rc_type rcode)
 	if (rcode == NSD_RC_DISCARD) {
 		return QUERY_DISCARDED;
 	}
-	
+
 	buffer_clear(q->packet);
-	
+
 	QR_SET(q->packet);	   /* This is an answer.  */
 	RCODE_SET(q->packet, (int) rcode); /* Error code.  */
-	
+
 	/* Truncate the question as well... */
 	QDCOUNT_SET(q->packet, 0);
 	ANCOUNT_SET(q->packet, 0);
@@ -178,18 +178,18 @@ query_create(region_type *region, uint16_t *compressed_dname_offsets,
 	return query;
 }
 
-void 
+void
 query_reset(query_type *q, size_t maxlen, int is_tcp)
 {
-	/* 
+	/*
 	 * As long as less than 4Kb (region block size) has been used,
 	 * this call to free_all is free, the block is saved for re-use,
-	 * so no malloc() or free() calls are done. 
+	 * so no malloc() or free() calls are done.
 	 * at present use of the region is for:
 	 *   o query qname dname_type (255 max).
 	 *   o wildcard expansion domain_type (7*ptr+u32+2bytes)+(5*ptr nsec3)
 	 *   o wildcard expansion for additional section domain_type.
-	 *   o nsec3 hashed name(s) (3 dnames for a nonexist_proof, 
+	 *   o nsec3 hashed name(s) (3 dnames for a nonexist_proof,
 	 *     one proof per wildcard and for nx domain).
 	 */
 	region_free_all(q->region);
@@ -233,12 +233,12 @@ query_get_tempdomain(struct query *q)
 		return 0;
 	q->number_temporary_domains ++;
 	memset(&d[q->number_temporary_domains-1], 0, sizeof(domain_type));
-	d[q->number_temporary_domains-1].number = q->compressed_dname_offsets_size + 
+	d[q->number_temporary_domains-1].number = q->compressed_dname_offsets_size +
 		q->number_temporary_domains - 1;
 	return &d[q->number_temporary_domains-1];
 }
 
-static void 
+static void
 query_addtxt(struct query  *q,
 	     const uint8_t *dname,
 	     uint16_t       klass,
@@ -249,7 +249,7 @@ query_addtxt(struct query  *q,
 	uint8_t len = (uint8_t) txt_length;
 
 	assert(txt_length <= UCHAR_MAX);
-	
+
 	/* Add the dname */
 	if (dname >= buffer_begin(q->packet)
 	    && dname <= buffer_current(q->packet))
@@ -303,6 +303,7 @@ process_edns(struct query *q)
 	if (q->edns.status == EDNS_ERROR) {
 		return NSD_RC_FORMAT;
 	}
+
 	if (q->edns.status == EDNS_OK) {
 		/* Only care about UDP size larger than normal... */
 		if (!q->tcp && q->edns.maxlen > UDP_MAX_MESSAGE_LEN) {
@@ -311,7 +312,7 @@ process_edns(struct query *q)
 			} else {
 				q->maxlen = EDNS_MAX_MESSAGE_LEN;
 			}
-			
+
 #if defined(INET6) && !defined(IPV6_USE_MIN_MTU)
 			/*
 			 * Use IPv6 minimum MTU to avoid sending
@@ -326,7 +327,7 @@ process_edns(struct query *q)
 			}
 #endif
 		}
-		
+
 		/* Strip the OPT resource record off... */
 		buffer_set_position(q->packet, q->edns.position);
 		buffer_set_limit(q->packet, q->edns.position);
@@ -370,7 +371,7 @@ process_tsig(struct query* q)
  * Check notify acl and forward to xfrd (or return an error).
  */
 static query_state_type
-answer_notify (struct nsd* nsd, struct query *query)
+answer_notify(struct nsd* nsd, struct query *query)
 {
 	int acl_num;
 	acl_options_t *why;
@@ -381,12 +382,12 @@ answer_notify (struct nsd* nsd, struct query *query)
 		dname_to_string(query->qname, NULL)));
 
 	zone_opt = zone_options_find(nsd->options, query->qname);
-	if(!zone_opt) 
+	if(!zone_opt)
 		return query_error(query, NSD_RC_NXDOMAIN);
-	
+
 	if(!nsd->this_child) /* we are in debug mode or something */
 		return query_error(query, NSD_RC_SERVFAIL);
-	
+
 #ifdef TSIG
 	if(!tsig_find_rr(&query->tsig, query->packet)) {
 		DEBUG(DEBUG_XFRD,2, (LOG_ERR, "bad tsig RR format"));
@@ -409,7 +410,7 @@ answer_notify (struct nsd* nsd, struct query *query)
 		assert(why);
 		DEBUG(DEBUG_XFRD,1, (LOG_INFO, "got notify %s passed acl %s %s",
 			dname_to_string(query->qname, NULL),
-			why->ip_address_spec, 
+			why->ip_address_spec,
 			why->nokey?"NOKEY":
 			(why->blocked?"BLOCKED":why->key_name)));
 		sz = buffer_limit(query->packet);
@@ -418,7 +419,7 @@ answer_notify (struct nsd* nsd, struct query *query)
 		/* forward to xfrd for processing
 		   Note. Blocking IPC I/O, but acl is OK. */
 		sz = htons(sz);
-		if(!write_socket(s, &mode, sizeof(mode)) || 
+		if(!write_socket(s, &mode, sizeof(mode)) ||
 			!write_socket(s, &sz, sizeof(sz)) ||
 			!write_socket(s, buffer_begin(query->packet),
 				buffer_limit(query->packet)) ||
@@ -439,13 +440,13 @@ answer_notify (struct nsd* nsd, struct query *query)
 		pos = buffer_position(query->packet);
 		buffer_clear(query->packet);
 		buffer_set_position(query->packet, pos);
-		VERBOSITY(2, (LOG_INFO, "Notify received and accepted, forward to xfrd")); 
+		VERBOSITY(2, (LOG_INFO, "Notify received and accepted, forward to xfrd"));
 		/* tsig is added in add_additional later (if needed) */
 		return QUERY_PROCESSED;
 	}
 	VERBOSITY(1, (LOG_INFO, "got notify for zone: %s; Refused by acl: %s %s",
 			dname_to_string(query->qname, NULL),
-			why?why->key_name:"no acl matches", 
+			why?why->key_name:"no acl matches",
 			why?why->ip_address_spec:"."));
 	return query_error(query, NSD_RC_REFUSE);
 }
@@ -462,7 +463,7 @@ answer_chaos(struct nsd *nsd, query_type *q)
 	case TYPE_ANY:
 	case TYPE_TXT:
 		if ((q->qname->name_size == 11
-		     && memcmp(dname_name(q->qname), "\002id\006server", 11) == 0) || 
+		     && memcmp(dname_name(q->qname), "\002id\006server", 11) == 0) ||
 		    (q->qname->name_size ==  15
 		     && memcmp(dname_name(q->qname), "\010hostname\004bind", 15) == 0))
 		{
@@ -478,13 +479,17 @@ answer_chaos(struct nsd *nsd, query_type *q)
 			   (q->qname->name_size == 14
 			    && memcmp(dname_name(q->qname), "\007version\004bind", 14) == 0))
 		{
-			/* Add version */
-			query_addtxt(q,
+			if(!nsd->options->hide_version) {
+				/* Add version */
+				query_addtxt(q,
 				     buffer_begin(q->packet) + QHEADERSZ,
 				     CLASS_CH,
 				     0,
 				     nsd->version);
-			ANCOUNT_SET(q->packet, ANCOUNT(q->packet) + 1);
+				ANCOUNT_SET(q->packet, ANCOUNT(q->packet) + 1);
+			} else {
+				RCODE_SET(q->packet, RCODE_REFUSE);
+			}
 		}
 		break;
 	default:
@@ -512,7 +517,7 @@ find_covering_nsec(domain_type *closest_match,
 
 	/* loop away temporary created domains. For real ones it is &RBTREE_NULL */
 	while (closest_match->node.parent == NULL)
-		closest_match = closest_match->parent; 
+		closest_match = closest_match->parent;
 	while (closest_match) {
 		*nsec_rrset = domain_find_rrset(closest_match, zone, TYPE_NSEC);
 		if (*nsec_rrset) {
@@ -554,22 +559,22 @@ add_additional_rrsets(struct query *query, answer_type *answer,
 		      int allow_glue, struct additional_rr_types types[])
 {
 	size_t i;
-	
+
 	assert(query);
 	assert(answer);
 	assert(master_rrset);
 	assert(rdata_atom_is_domain(rrset_rrtype(master_rrset), rdata_index));
-	
+
 	for (i = 0; i < master_rrset->rr_count; ++i) {
 		int j;
 		domain_type *additional = rdata_atom_domain(master_rrset->rrs[i].rdatas[rdata_index]);
 		domain_type *match = additional;
-		
+
 		assert(additional);
 
 		if (!allow_glue && domain_is_glue(match, query->zone))
 			continue;
-		
+
 		/*
 		 * Check to see if we need to generate the dependent
 		 * based on a wildcard domain.
@@ -609,13 +614,13 @@ add_rrset(struct query   *query,
 	  rrset_type     *rrset)
 {
 	int result;
-	
+
 	assert(query);
 	assert(answer);
 	assert(owner);
 	assert(rrset);
 	assert(rrset_rrclass(rrset) == CLASS_IN);
-	
+
 	result = answer_add_rrset(answer, section, owner, rrset);
 	switch (rrset_rrtype(rrset)) {
 	case TYPE_NS:
@@ -648,7 +653,7 @@ add_rrset(struct query   *query,
    DNAME rr is from src to dest.
    closest encloser encloses the to_name. */
 static uint32_t
-query_synthesize_cname(struct query* q, struct answer* answer, const dname_type* from_name, 
+query_synthesize_cname(struct query* q, struct answer* answer, const dname_type* from_name,
 	const dname_type* to_name, domain_type* src, domain_type* to_closest_encloser,
 	domain_type** to_closest_match)
 {
@@ -667,7 +672,7 @@ query_synthesize_cname(struct query* q, struct answer* answer, const dname_type*
 	for(i=0; i < from_name->label_count - domain_dname(src)->label_count; i++)
 	{
 		domain_type* newdom = query_get_tempdomain(q);
-		if(!newdom) 
+		if(!newdom)
 			return 0;
 		newdom->is_existing = 1;
 		newdom->parent = lastparent;
@@ -686,11 +691,11 @@ query_synthesize_cname(struct query* q, struct answer* answer, const dname_type*
 
 	/* allocate dest part */
 	lastparent = to_closest_encloser;
-	for(i=0; i < to_name->label_count - domain_dname(to_closest_encloser)->label_count; 
+	for(i=0; i < to_name->label_count - domain_dname(to_closest_encloser)->label_count;
 		i++)
 	{
 		domain_type* newdom = query_get_tempdomain(q);
-		if(!newdom) 
+		if(!newdom)
 			return 0;
 		newdom->is_existing = 0;
 		newdom->parent = lastparent;
@@ -739,7 +744,7 @@ answer_delegation(query_type *query, answer_type *answer)
 	assert(answer);
 	assert(query->delegation_domain);
 	assert(query->delegation_rrset);
-	
+
 	AA_CLR(query->packet);
 	add_rrset(query,
 		  answer,
@@ -771,7 +776,7 @@ static void
 answer_soa(struct query *query, answer_type *answer)
 {
 	query->domain = query->zone->apex;
-	
+
 	if (query->qclass != CLASS_ANY) {
 		add_rrset(query, answer,
 			  AUTHORITY_SECTION,
@@ -796,11 +801,11 @@ answer_nodata(struct query *query, answer_type *answer, domain_type *original)
 	if (query->cname_count == 0) {
 		answer_soa(query, answer);
 	}
-	
+
 #ifdef NSEC3
 	if (query->edns.dnssec_ok && query->zone->nsec3_soa_rr) {
 		nsec3_answer_nodata(query, answer, original);
-	} else 
+	} else
 #endif
 	if (query->edns.dnssec_ok && zone_is_secure(query->zone)) {
 		domain_type *nsec_domain;
@@ -832,7 +837,7 @@ answer_domain(struct nsd* nsd, struct query *q, answer_type *answer,
 	      domain_type *domain, domain_type *original)
 {
 	rrset_type *rrset;
-	
+
 	if (q->qtype == TYPE_ANY) {
 		int added = 0;
 		for (rrset = domain_find_any_rrset(domain, q->zone); rrset; rrset = rrset->next) {
@@ -880,10 +885,10 @@ answer_domain(struct nsd* nsd, struct query *q, answer_type *answer,
 			domain_type *closest_encloser = closest_match;
 			zone_type* origzone = q->zone;
 			++q->cname_count;
-				
+
 			while (!closest_encloser->is_existing)
 				closest_encloser = closest_encloser->parent;
-				
+
 			answer_lookup_zone(nsd, q, answer, closest_match->number,
 					     closest_match == closest_encloser,
 					     closest_match, closest_encloser,
@@ -896,7 +901,7 @@ answer_domain(struct nsd* nsd, struct query *q, answer_type *answer,
 	}
 
 	q->domain = domain;
-	
+
 	if (q->qclass != CLASS_ANY && q->zone->ns_rrset) {
 		add_rrset(q, answer, AUTHORITY_SECTION, q->zone->apex,
 			  q->zone->ns_rrset);
@@ -926,7 +931,7 @@ answer_authoritative(struct nsd   *nsd,
 	domain_type *match;
 	domain_type *original = closest_match;
 	rrset_type *rrset;
-	
+
 #ifdef NSEC3
 	if(exact && domain_has_only_NSEC3(closest_match, q->zone)) {
 		exact = 0; /* pretend it does not exist */
@@ -946,15 +951,15 @@ answer_authoritative(struct nsd   *nsd,
 		if(domain_number != 0) /* we followed CNAMEs or DNAMEs */
 			name = domain_dname(closest_match);
 		DEBUG(DEBUG_QUERY,2, (LOG_INFO, "expanding DNAME for q=%s", dname_to_string(name, NULL)));
-		DEBUG(DEBUG_QUERY,2, (LOG_INFO, "->src is %s", 
+		DEBUG(DEBUG_QUERY,2, (LOG_INFO, "->src is %s",
 			dname_to_string(domain_dname(closest_encloser), NULL)));
-		DEBUG(DEBUG_QUERY,2, (LOG_INFO, "->dest is %s", 
+		DEBUG(DEBUG_QUERY,2, (LOG_INFO, "->dest is %s",
 			dname_to_string(domain_dname(dest), NULL)));
 		/* if the DNAME set is not added we have a loop, do not follow */
 		added = add_rrset(q, answer, ANSWER_SECTION, closest_encloser, rrset);
 		if(added) {
 			domain_type* src = closest_encloser;
-			const dname_type* newname = dname_replace(q->region, name, 
+			const dname_type* newname = dname_replace(q->region, name,
 				domain_dname(src), domain_dname(dest));
 			uint32_t newnum = 0;
 			zone_type* origzone = q->zone;
@@ -967,7 +972,7 @@ answer_authoritative(struct nsd   *nsd,
 			/* follow the DNAME */
 			exact = namedb_lookup(nsd->db, newname, &closest_match, &closest_encloser);
 			/* synthesize CNAME record */
-			newnum = query_synthesize_cname(q, answer, name, newname, 
+			newnum = query_synthesize_cname(q, answer, name, newname,
 				src, closest_encloser, &closest_match);
 			if(!newnum) {
 				/* could not synthesize the CNAME. */
@@ -978,7 +983,7 @@ answer_authoritative(struct nsd   *nsd,
 			while (closest_encloser && !closest_encloser->is_existing)
 				closest_encloser = closest_encloser->parent;
 			answer_lookup_zone(nsd, q, answer, newnum,
-				closest_match == closest_encloser, 
+				closest_match == closest_encloser,
 				closest_match, closest_encloser, newname);
 			q->zone = origzone;
 		}
@@ -1004,7 +1009,11 @@ answer_authoritative(struct nsd   *nsd,
 		match->nsec3_wcard_child_cover = wildcard_child->nsec3_wcard_child_cover;
 		match->nsec3_ds_parent_is_exact = wildcard_child->nsec3_ds_parent_is_exact;
 		match->nsec3_ds_parent_cover = wildcard_child->nsec3_ds_parent_cover;
-		nsec3_answer_wildcard(q, answer, wildcard_child, nsd->db, qname);
+
+		if (q->edns.dnssec_ok && q->zone->nsec3_soa_rr) {
+			/* Only add nsec3 wildcard data when do bit is set */
+			nsec3_answer_wildcard(q, answer, wildcard_child, nsd->db, qname);
+		}
 #endif
 
 		/*
@@ -1021,15 +1030,15 @@ answer_authoritative(struct nsd   *nsd,
 	/* Authorative zone.  */
 #ifdef NSEC3
 	if (q->edns.dnssec_ok && q->zone->nsec3_soa_rr) {
-		nsec3_answer_authoritative(&match, q, answer, 
+		nsec3_answer_authoritative(&match, q, answer,
 			closest_encloser, nsd->db, qname);
-	} else 
+	} else
 #endif
 	if (q->edns.dnssec_ok && zone_is_secure(q->zone)) {
 		if (match != closest_encloser) {
 			domain_type *nsec_domain;
 			rrset_type *nsec_rrset;
-			
+
 			/*
 			 * No match found or generated from wildcard,
 			 * include NSEC record.
@@ -1053,7 +1062,7 @@ answer_authoritative(struct nsd   *nsd,
 			}
 		}
 	}
-	
+
 #ifdef NSEC3
 	if (RCODE(q->packet)!=RCODE_OK) {
 		return; /* nsec3 collision failure */
@@ -1071,9 +1080,9 @@ answer_authoritative(struct nsd   *nsd,
  */
 static void
 answer_lookup_zone(struct nsd *nsd, struct query *q, answer_type *answer,
-	uint32_t domain_number, int exact, domain_type *closest_match, 
+	uint32_t domain_number, int exact, domain_type *closest_match,
 	domain_type *closest_encloser, const dname_type *qname)
-{	
+{
 	q->zone = domain_find_zone(closest_encloser);
 	if (!q->zone) {
 		if(q->cname_count == 0)
@@ -1159,14 +1168,17 @@ answer_query(struct nsd *nsd, struct query *q)
 	}
 
 	q->domain = closest_encloser;
-	answer_lookup_zone(nsd, q, &answer, 0, exact, closest_match, 
+	answer_lookup_zone(nsd, q, &answer, 0, exact, closest_match,
 		closest_encloser, q->qname);
 
+	encode_answer(q, &answer);
+	if (ANCOUNT(q->packet) + NSCOUNT(q->packet) + ARCOUNT(q->packet) == 0)
+	{
+		/* no answers, no need for compression */
+		return;
+	}
 	offset = dname_label_offsets(q->qname)[domain_dname(closest_encloser)->label_count - 1] + QHEADERSZ;
 	query_add_compression_domain(q, closest_encloser, offset);
-
-	encode_answer(q, &answer);
-
 	query_clear_compression_tables(q);
 }
 
@@ -1174,13 +1186,13 @@ void
 query_prepare_response(query_type *q)
 {
 	uint16_t flags;
-	
+
 	/*
 	 * Preserve the data up-to the current packet's limit.
 	 */
 	buffer_set_position(q->packet, buffer_limit(q->packet));
 	buffer_set_limit(q->packet, buffer_capacity(q->packet));
-	
+
 	/*
 	 * Reserve space for the EDNS records if required.
 	 */
@@ -1188,7 +1200,7 @@ query_prepare_response(query_type *q)
 #ifdef TSIG
 	q->reserved_space += tsig_reserved_space(&q->tsig);
 #endif /* TSIG */
-	
+
 	/* Update the flags.  */
 	flags = FLAGS(q->packet);
 	flags &= 0x0100U;	/* Preserve the RD flag.  */
@@ -1211,7 +1223,7 @@ query_process(query_type *q, nsd_type *nsd)
 
 	/* Sanity checks */
 	if (buffer_limit(q->packet) < QHEADERSZ) {
-		/* packet too small to contain DNS header. 
+		/* packet too small to contain DNS header.
 		Now packet investigation macros will work without problems. */
 		return QUERY_DISCARDED;
 	}
@@ -1243,15 +1255,15 @@ query_process(query_type *q, nsd_type *nsd)
 		return query_formerr(q);
 	}
 
-	/* Dont allow any records in the answer or authority section... 
+	/* Dont allow any records in the answer or authority section...
 	   except for IXFR queries. */
-	if (ANCOUNT(q->packet) != 0 || 
+	if (ANCOUNT(q->packet) != 0 ||
 		(q->qtype!=TYPE_IXFR && NSCOUNT(q->packet) != 0)) {
 		return query_formerr(q);
 	}
 	if(q->qtype==TYPE_IXFR && NSCOUNT(q->packet) > 0) {
 		int i; /* skip ixfr soa information data here */
-		for(i=0; i<NSCOUNT(q->packet); i++)
+		for(i=0; i< NSCOUNT(q->packet); i++)
 			if(!packet_skip_rr(q->packet, 0))
 				return query_formerr(q);
 	}
@@ -1292,7 +1304,7 @@ query_process(query_type *q, nsd_type *nsd)
 #endif
 	/* Remove trailing garbage.  */
 	buffer_set_limit(q->packet, buffer_position(q->packet));
-	
+
 #ifdef TSIG
 	rc = process_tsig(q);
 	if (rc != NSD_RC_OK) {
@@ -1301,11 +1313,15 @@ query_process(query_type *q, nsd_type *nsd)
 #endif /* TSIG */
 	rc = process_edns(q);
 	if (rc != NSD_RC_OK) {
-		return query_error(q, rc);
+		/* We should not return FORMERR, but BADVERS (=16).
+		 * BADVERS is created with Ext. RCODE, followed by RCODE.
+		 * Ext. RCODE is set to 1, RCODE must be 0 (getting 0x10 = 16).
+		 * Thus RCODE = NOERROR = NSD_RC_OK. */
+		return query_error(q, NSD_RC_OK);
 	}
 
 	query_prepare_response(q);
-	
+
 	if (q->qclass != CLASS_IN && q->qclass != CLASS_ANY) {
 		if (q->qclass == CLASS_CH) {
 			return answer_chaos(nsd, q);
@@ -1355,27 +1371,27 @@ query_add_optional(query_type *q, nsd_type *nsd)
 #else
 		buffer_write(q->packet, edns->rdata_none, OPT_RDATA);
 #endif /* NSID */
-		
+
 		ARCOUNT_SET(q->packet, ARCOUNT(q->packet) + 1);
 
 		STATUP(nsd, edns);
 		break;
 	case EDNS_ERROR:
 		buffer_write(q->packet, edns->error, OPT_LEN);
+		buffer_write(q->packet, edns->rdata_none, OPT_RDATA);
 		ARCOUNT_SET(q->packet, ARCOUNT(q->packet) + 1);
-
 		STATUP(nsd, ednserr);
 		break;
 	}
 
 #ifdef TSIG
 	if (q->tsig.status != TSIG_NOT_PRESENT) {
-		if (q->tsig.status == TSIG_ERROR || 
+		if (q->tsig.status == TSIG_ERROR ||
 			q->tsig.error_code != TSIG_ERROR_NOERROR) {
 			tsig_error_reply(&q->tsig);
 			tsig_append_rr(&q->tsig, q->packet);
 			ARCOUNT_SET(q->packet, ARCOUNT(q->packet) + 1);
-		} else if(q->tsig.status == TSIG_OK && 
+		} else if(q->tsig.status == TSIG_OK &&
 			q->tsig.error_code == TSIG_ERROR_NOERROR)
 		{
 			if(q->tsig_prepare_it)
