@@ -43,6 +43,11 @@ void log_init(const char *ident);
  */
 void log_open(int option, int facility, const char *filename);
 
+/*
+ * Reopen the logfile.
+ */
+void log_reopen(const char *filename, uint8_t verbose);
+
 /*
  * Finalize the logging system.
  */
@@ -208,7 +213,7 @@ read_uint32(const void *src)
 }
 
 /*
- * Print debugging information using log_msg, 
+ * Print debugging information using log_msg,
  * set the logfile as /dev/stdout or /dev/stderr if you like.
  * nsd -F 0xFFFF enables all debug facilities.
  */
@@ -220,11 +225,11 @@ read_uint32(const void *src)
 #define DEBUG_XFRD             0x0020U
 #define DEBUG_IPC              0x0040U
 
+extern unsigned nsd_debug_facilities;
+extern int nsd_debug_level;
 #ifdef NDEBUG
 #define DEBUG(facility, level, args)  /* empty */
 #else
-extern unsigned nsd_debug_facilities;
-extern int nsd_debug_level;
 #define DEBUG(facility, level, args)				\
 	do {							\
 		if ((facility) & nsd_debug_facilities &&	\
@@ -267,7 +272,7 @@ timeval_to_timespec(struct timespec *left,
  * XXX This functions does not check the range.
  *
  */
-long strtottl(const char *nptr, const char **endptr);
+uint32_t strtottl(const char *nptr, const char **endptr);
 
 /*
  * Convert binary data to a string of hexadecimal characters.
@@ -320,7 +325,7 @@ int compare_serial(uint32_t a, uint32_t b);
  */
 void cleanup_region(void *data);
 
-/* 
+/*
  * Region used to store owner and origin of previous RR (used
  * for pretty printing of zone data).
  * Keep the same between calls to print_rr.
@@ -339,7 +344,7 @@ int print_rr(FILE *out, struct state_pretty_rr* state, struct rr *record);
  */
 const char* rcode2str(int rc);
 
-/* 
+/*
  * Stack of pointers.
  * Stack is fixed size on start. More elems fall off stack.
  */
