@@ -35,56 +35,56 @@ xfrd_tcp_set_t* xfrd_tcp_set_create(struct region* region)
 	return tcp_set;
 }
 
-void 
+void
 xfrd_setup_packet(buffer_type* packet,
 	uint16_t type, uint16_t klass, const dname_type* dname)
-{	
+{
 	/* Set up the header */
 	buffer_clear(packet);
 	ID_SET(packet, (uint16_t) random());
-        FLAGS_SET(packet, 0);
-        OPCODE_SET(packet, OPCODE_QUERY);
-        QDCOUNT_SET(packet, 1);
-        ANCOUNT_SET(packet, 0);
-        NSCOUNT_SET(packet, 0);
-        ARCOUNT_SET(packet, 0);
-        buffer_skip(packet, QHEADERSZ);
+	FLAGS_SET(packet, 0);
+	OPCODE_SET(packet, OPCODE_QUERY);
+	QDCOUNT_SET(packet, 1);
+	ANCOUNT_SET(packet, 0);
+	NSCOUNT_SET(packet, 0);
+	ARCOUNT_SET(packet, 0);
+	buffer_skip(packet, QHEADERSZ);
 
 	/* The question record. */
-        buffer_write(packet, dname_name(dname), dname->name_size);
-        buffer_write_u16(packet, type);
-        buffer_write_u16(packet, klass);
+	buffer_write(packet, dname_name(dname), dname->name_size);
+	buffer_write_u16(packet, type);
+	buffer_write_u16(packet, klass);
 }
 
-socklen_t 
+static socklen_t
 #ifdef INET6
-xfrd_acl_sockaddr(acl_options_t* acl, struct sockaddr_storage *to)
+xfrd_acl_sockaddr(acl_options_t* acl, unsigned int port,
+	struct sockaddr_storage *sck)
 #else
-xfrd_acl_sockaddr(acl_options_t* acl, struct sockaddr_in *to)
+xfrd_acl_sockaddr(acl_options_t* acl, unsigned int port,
+	struct sockaddr_in *sck, const char* fromto)
 #endif /* INET6 */
 {
-	unsigned int port = acl->port?acl->port:(unsigned)atoi(TCP_PORT);
-
 	/* setup address structure */
 #ifdef INET6
-	memset(to, 0, sizeof(struct sockaddr_storage));
+	memset(sck, 0, sizeof(struct sockaddr_storage));
 #else
-	memset(to, 0, sizeof(struct sockaddr_in));
+	memset(sck, 0, sizeof(struct sockaddr_in));
 #endif
 	if(acl->is_ipv6) {
 #ifdef INET6
-		struct sockaddr_in6* sa = (struct sockaddr_in6*)to;
+		struct sockaddr_in6* sa = (struct sockaddr_in6*)sck;
 		sa->sin6_family = AF_INET6;
 		sa->sin6_port = htons(port);
 		sa->sin6_addr = acl->addr.addr6;
 		return sizeof(struct sockaddr_in6);
 #else
-		log_msg(LOG_ERR, "xfrd: IPv6 connection to %s attempted but no INET6.",
-			acl->ip_address_spec);
+		log_msg(LOG_ERR, "xfrd: IPv6 connection %s %s attempted but no \
+INET6.", fromto, acl->ip_address_spec);
 		return 0;
 #endif
 	} else {
-		struct sockaddr_in* sa = (struct sockaddr_in*)to;
+		struct sockaddr_in* sa = (struct sockaddr_in*)sck;
 		sa->sin_family = AF_INET;
 		sa->sin_port = htons(port);
 		sa->sin_addr = acl->addr.addr;
@@ -92,7 +92,37 @@ xfrd_acl_sockaddr(acl_options_t* acl, struct sockaddr_in *to)
 	}
 }
 
-void 
+socklen_t
+#ifdef INET6
+xfrd_acl_sockaddr_to(acl_options_t* acl, struct sockaddr_storage *to)
+#else
+xfrd_acl_sockaddr_to(acl_options_t* acl, struct sockaddr_in *to)
+#endif /* INET6 */
+{
+	unsigned int port = acl->port?acl->port:(unsigned)atoi(TCP_PORT);
+#ifdef INET6
+	return xfrd_acl_sockaddr(acl, port, to);
+#else
+	return xfrd_acl_sockaddr(acl, port, to, "to");
+#endif /* INET6 */
+}
+
+socklen_t
+#ifdef INET6
+xfrd_acl_sockaddr_frm(acl_options_t* acl, struct sockaddr_storage *frm)
+#else
+xfrd_acl_sockaddr_frm(acl_options_t* acl, struct sockaddr_in *frm)
+#endif /* INET6 */
+{
+	unsigned int port = acl->port?acl->port:0;
+#ifdef INET6
+	return xfrd_acl_sockaddr(acl, port, frm);
+#else
+	return xfrd_acl_sockaddr(acl, port, frm, "from");
+#endif /* INET6 */
+}
+
+void
 xfrd_write_soa_buffer(struct buffer* packet,
 	const dname_type* apex, struct xfrd_soa* soa)
 {
@@ -122,7 +152,7 @@ xfrd_write_soa_buffer(struct buffer* packet,
 	buffer_write_u16_at(packet, rdlength_pos, rdlength);
 }
 
-xfrd_tcp_t* 
+xfrd_tcp_t*
 xfrd_tcp_create(region_type* region)
 {
 	xfrd_tcp_t* tcp_state = (xfrd_tcp_t*)region_alloc(
@@ -134,7 +164,7 @@ xfrd_tcp_create(region_type* region)
 	return tcp_state;
 }
 
-void 
+void
 xfrd_tcp_obtain(xfrd_tcp_set_t* set, xfrd_zone_t* zone)
 {
 	assert(zone->tcp_conn == -1);
@@ -160,7 +190,7 @@ xfrd_tcp_obtain(xfrd_tcp_set_t* set, xfrd_zone_t* zone)
 		if(zone->zone_handler.fd != -1)
 			xfrd_udp_release(zone);
 
-		if(!xfrd_tcp_open(set, zone)) 
+		if(!xfrd_tcp_open(set, zone))
 			return;
 
 		xfrd_tcp_xfr(set, zone);
@@ -181,12 +211,10 @@ xfrd_tcp_obtain(xfrd_tcp_set_t* set, xfrd_zone_t* zone)
 	xfrd_unset_timer(zone);
 }
 
-int 
+int
 xfrd_tcp_open(xfrd_tcp_set_t* set, xfrd_zone_t* zone)
 {
-	/* TODO use port 53 */
-	int fd;
-	int family;
+	int fd, family, conn;
 
 #ifdef INET6
 	struct sockaddr_storage to;
@@ -210,12 +238,13 @@ xfrd_tcp_open(xfrd_tcp_set_t* set, xfrd_zone_t* zone)
 		xfrd_tcp_release(set, zone);
 		return 0;
 #endif
-	} else { 
+	} else {
 		family = PF_INET;
 	}
 	fd = socket(family, SOCK_STREAM, IPPROTO_TCP);
+	set->tcp_state[zone->tcp_conn]->fd = fd;
 	if(fd == -1) {
-		log_msg(LOG_ERR, "xfrd: %s cannot create tcp socket: %s", 
+		log_msg(LOG_ERR, "xfrd: %s cannot create tcp socket: %s",
 			zone->master->ip_address_spec, strerror(errno));
 		xfrd_set_refresh_now(zone);
 		xfrd_tcp_release(set, zone);
@@ -228,35 +257,51 @@ xfrd_tcp_open(xfrd_tcp_set_t* set, xfrd_zone_t* zone)
 		return 0;
 	}
 
-	to_len = xfrd_acl_sockaddr(zone->master, &to);
-	if(connect(fd, (struct sockaddr*)&to, to_len) == -1)
-	{
-		if(errno != EINPROGRESS) {
-			log_msg(LOG_ERR, "xfrd: connect %s failed: %s",
-				zone->master->ip_address_spec, strerror(errno));
-			xfrd_set_refresh_now(zone);
-			xfrd_tcp_release(set, zone);
-			return 0;
-		}
+	to_len = xfrd_acl_sockaddr_to(zone->master, &to);
+
+	/* bind it */
+	if (!xfrd_bind_local_interface(fd,
+		zone->zone_options->outgoing_interface, zone->master, 1)) {
+
+		xfrd_set_refresh_now(zone);
+		xfrd_tcp_release(set, zone);
+		return 0;
+        }
+
+	conn = connect(fd, (struct sockaddr*)&to, to_len);
+	if (conn == -1 && errno != EINPROGRESS) {
+		log_msg(LOG_ERR, "xfrd: connect %s failed: %s",
+			zone->master->ip_address_spec, strerror(errno));
+		xfrd_set_refresh_now(zone);
+		xfrd_tcp_release(set, zone);
+		return 0;
 	}
 
 	zone->zone_handler.fd = fd;
-	set->tcp_state[zone->tcp_conn]->fd = fd;
 	zone->zone_handler.event_types = NETIO_EVENT_TIMEOUT|NETIO_EVENT_WRITE;
 	xfrd_set_timer(zone, xfrd_time() + XFRD_TCP_TIMEOUT);
 	return 1;
 }
 
-void 
+void
 xfrd_tcp_xfr(xfrd_tcp_set_t* set, xfrd_zone_t* zone)
 {
 	xfrd_tcp_t* tcp = set->tcp_state[zone->tcp_conn];
 	assert(zone->tcp_conn != -1);
 	assert(zone->tcp_waiting == 0);
 	/* start AXFR or IXFR for the zone */
-	if(zone->soa_disk_acquired == 0 || zone->master->use_axfr_only) {
+	if(zone->soa_disk_acquired == 0 || zone->master->use_axfr_only ||
+						zone->master->ixfr_disabled) {
+		DEBUG(DEBUG_XFRD,1, (LOG_INFO, "request full zone transfer "
+						"(AXFR) for %s to %s",
+			zone->apex_str, zone->master->ip_address_spec));
+
 		xfrd_setup_packet(tcp->packet, TYPE_AXFR, CLASS_IN, zone->apex);
 	} else {
+		DEBUG(DEBUG_XFRD,1, (LOG_INFO, "request incremental zone "
+						"transfer (IXFR) for %s to %s",
+			zone->apex_str, zone->master->ip_address_spec));
+
 		xfrd_setup_packet(tcp->packet, TYPE_IXFR, CLASS_IN, zone->apex);
         	NSCOUNT_SET(tcp->packet, 1);
 		xfrd_write_soa_buffer(tcp->packet, zone->apex, &zone->soa_disk);
@@ -264,11 +309,11 @@ xfrd_tcp_xfr(xfrd_tcp_set_t* set, xfrd_zone_t* zone)
 	zone->query_id = ID(tcp->packet);
 	zone->msg_seq_nr = 0;
 	zone->msg_rr_count = 0;
-	if(zone->master->key_options) {
 #ifdef TSIG
+	if(zone->master->key_options && zone->master->key_options->tsig_key) {
 		xfrd_tsig_sign_request(tcp->packet, &zone->tsig, zone->master);
-#endif /* TSIG */
 	}
+#endif /* TSIG */
 	buffer_flip(tcp->packet);
 	DEBUG(DEBUG_XFRD,1, (LOG_INFO, "sent tcp query with ID %d", zone->query_id));
 	tcp->msglen = buffer_limit(tcp->packet);
@@ -289,7 +334,7 @@ int conn_write(xfrd_tcp_t* tcp)
 
 	if(tcp->total_bytes < sizeof(tcp->msglen)) {
 		uint16_t sendlen = htons(tcp->msglen);
-		sent = write(tcp->fd, 
+		sent = write(tcp->fd,
 			(const char*)&sendlen + tcp->total_bytes,
 			sizeof(tcp->msglen) - tcp->total_bytes);
 
@@ -311,7 +356,7 @@ int conn_write(xfrd_tcp_t* tcp)
 	}
 
 	assert(tcp->total_bytes < tcp->msglen + sizeof(tcp->msglen));
-	
+
 	sent = write(tcp->fd,
 		buffer_current(tcp->packet),
 		buffer_remaining(tcp->packet));
@@ -323,7 +368,7 @@ int conn_write(xfrd_tcp_t* tcp)
 			return -1;
 		}
 	}
-	
+
 	buffer_skip(tcp->packet, sent);
 	tcp->total_bytes += sent;
 
@@ -331,12 +376,12 @@ int conn_write(xfrd_tcp_t* tcp)
 		/* more to write when socket becomes writable again */
 		return 0;
 	}
-	
+
 	assert(tcp->total_bytes == tcp->msglen + sizeof(tcp->msglen));
 	return 1;
 }
 
-void 
+void
 xfrd_tcp_write(xfrd_tcp_set_t* set, xfrd_zone_t* zone)
 {
 	int ret;
@@ -377,7 +422,8 @@ xfrd_tcp_write(xfrd_tcp_set_t* set, xfrd_zone_t* zone)
 	xfrd_tcp_read(set, zone);
 }
 
-int conn_read(xfrd_tcp_t* tcp)
+int
+conn_read(xfrd_tcp_t* tcp)
 {
 	ssize_t received;
 	/* receive leading packet length bytes */
@@ -390,6 +436,9 @@ int conn_read(xfrd_tcp_t* tcp)
 				/* read would block, try later */
 				return 0;
 			} else {
+#ifdef ECONNRESET
+				if (verbosity >= 2 || errno != ECONNRESET)
+#endif /* ECONNRESET */
 				log_msg(LOG_ERR, "tcp read sz: %s", strerror(errno));
 				return -1;
 			}
@@ -415,15 +464,17 @@ int conn_read(xfrd_tcp_t* tcp)
 
 	assert(buffer_remaining(tcp->packet) > 0);
 
-	received = read(tcp->fd, buffer_current(tcp->packet), 
+	received = read(tcp->fd, buffer_current(tcp->packet),
 		buffer_remaining(tcp->packet));
 	if(received == -1) {
 		if(errno == EAGAIN || errno == EINTR) {
 			/* read would block, try later */
 			return 0;
 		} else {
-			log_msg(LOG_ERR, "tcp read %s",
-				strerror(errno));
+#ifdef ECONNRESET
+			if (verbosity >= 2 || errno != ECONNRESET)
+#endif /* ECONNRESET */
+			log_msg(LOG_ERR, "tcp read %s", strerror(errno));
 			return -1;
 		}
 	} else if(received == 0) {
@@ -444,12 +495,12 @@ int conn_read(xfrd_tcp_t* tcp)
 	return 1;
 }
 
-void 
+void
 xfrd_tcp_read(xfrd_tcp_set_t* set, xfrd_zone_t* zone)
 {
 	xfrd_tcp_t* tcp = set->tcp_state[zone->tcp_conn];
 	int ret;
-	
+
 	assert(zone->tcp_conn != -1);
 	ret = conn_read(tcp);
 	if(ret == -1) {
@@ -457,7 +508,7 @@ xfrd_tcp_read(xfrd_tcp_set_t* set, xfrd_zone_t* zone)
 		xfrd_tcp_release(set, zone);
 		return;
 	}
-	if(ret == 0) 
+	if(ret == 0)
 		return;
 
 	/* completed msg */
@@ -471,6 +522,12 @@ xfrd_tcp_read(xfrd_tcp_set_t* set, xfrd_zone_t* zone)
 			xfrd_tcp_release(set, zone);
 			assert(zone->round_num == -1);
 			break;
+		case xfrd_packet_notimpl:
+			zone->master->ixfr_disabled = time(NULL);
+			xfrd_tcp_release(set, zone);
+			/* query next server */
+			xfrd_make_request(zone);
+			break;
 		case xfrd_packet_bad:
 		case xfrd_packet_tcp:
 		default:
@@ -481,7 +538,7 @@ xfrd_tcp_read(xfrd_tcp_set_t* set, xfrd_zone_t* zone)
 	}
 }
 
-void 
+void
 xfrd_tcp_release(xfrd_tcp_set_t* set, xfrd_zone_t* zone)
 {
 	int conn = zone->tcp_conn;
@@ -502,7 +559,7 @@ xfrd_tcp_release(xfrd_tcp_set_t* set, xfrd_zone_t* zone)
 	if(set->tcp_count == XFRD_MAX_TCP && set->tcp_waiting_first) {
 		/* pop first waiting process */
 		zone = set->tcp_waiting_first;
-		if(set->tcp_waiting_last == zone) 
+		if(set->tcp_waiting_last == zone)
 			set->tcp_waiting_last = 0;
 
 		set->tcp_waiting_first = zone->tcp_waiting_next;
@@ -515,7 +572,7 @@ xfrd_tcp_release(xfrd_tcp_set_t* set, xfrd_zone_t* zone)
 		if(zone->zone_handler.fd != -1)
 			xfrd_udp_release(zone);
 
-		if(!xfrd_tcp_open(set, zone)) 
+		if(!xfrd_tcp_open(set, zone))
 			return;
 
 		xfrd_tcp_xfr(set, zone);
