@@ -81,7 +81,7 @@ alloc_rdata_init(region_type *region, const void *data, size_t size)
 	return result;
 }
 
-/* 
+/*
  * These are parser function for generic zone file stuff.
  */
 uint16_t *
@@ -91,7 +91,7 @@ zparser_conv_hex(region_type *region, const char *hex, size_t len)
 	uint16_t *r = NULL;
 	uint8_t *t;
 	int i;
-	
+
 	if (len % 2 != 0) {
 		zc_error_prev_line("number of hex digits must be a multiple of 2");
 	} else if (len > MAX_RDLENGTH * 2) {
@@ -101,12 +101,12 @@ zparser_conv_hex(region_type *region, const char *hex, size_t len)
 		/* the length part */
 		r = alloc_rdata(region, len/2);
 		t = (uint8_t *)(r + 1);
-    
+
 		/* Now process octet by octet... */
 		while (*hex) {
 			*t = 0;
 			for (i = 16; i >= 1; i -= 15) {
-				if (isxdigit(*hex)) {
+				if (isxdigit((int)*hex)) {
 					*t += hexdigit_to_int(*hex) * i;
 				} else {
 					zc_error_prev_line(
@@ -135,19 +135,19 @@ zparser_conv_hex_length(region_type *region, const char *hex, size_t len)
 		zc_error_prev_line("hex data exceeds 255 bytes");
 	} else {
 		uint8_t *l;
-		
+
 		/* the length part */
 		r = alloc_rdata(region, len/2+1);
 		t = (uint8_t *)(r + 1);
-		
+
 		l = t++;
 		*l = '\0';
-		
+
 		/* Now process octet by octet... */
 		while (*hex) {
 			*t = 0;
 			for (i = 16; i >= 1; i -= 15) {
-				if (isxdigit(*hex)) {
+				if (isxdigit((int)*hex)) {
 					*t += hexdigit_to_int(*hex) * i;
 				} else {
 					zc_error_prev_line(
@@ -212,7 +212,7 @@ zparser_conv_services(region_type *region, const char *protostr,
 	for (word = strtok(servicestr, sep); word; word = strtok(NULL, sep)) {
 		struct servent *service;
 		int port;
-		
+
 		service = getservbyname(word, proto->p_name);
 		if (service) {
 			/* Note: ntohs not ntohl!  Strange but true.  */
@@ -240,7 +240,27 @@ zparser_conv_services(region_type *region, const char *protostr,
 	p = (uint8_t *) (r + 1);
 	*p = proto->p_proto;
 	memcpy(p + 1, bitmap, *r);
-	
+
+	return r;
+}
+
+uint16_t *
+zparser_conv_serial(region_type *region, const char *serialstr)
+{
+	uint16_t *r = NULL;
+	uint32_t serial;
+	long long v;
+	char *t;
+
+	v = strtoll(serialstr, &t, 10);
+	if (serialstr[0] == '\0' || *t != '\0' || v < 0 || v > UINT_MAX
+		|| (errno == ERANGE && (v == LLONG_MIN || v == LLONG_MAX)))
+	{
+		zc_error_prev_line("serial is expected");
+	} else {
+		serial = htonl((uint32_t) v);
+		r = alloc_rdata_init(region, &serial, sizeof(serial));
+	}
 	return r;
 }
 
@@ -250,10 +270,10 @@ zparser_conv_period(region_type *region, const char *periodstr)
 	/* convert a time period (think TTL's) to wireformat) */
 	uint16_t *r = NULL;
 	uint32_t period;
-	const char *end; 
+	const char *end;
 
 	/* Allocate required space... */
-	period = (uint32_t) strtottl(periodstr, &end);
+	period = strtottl(periodstr, &end);
 	if (*end != '\0') {
 		zc_error_prev_line("time period is expected");
 	} else {
@@ -269,7 +289,7 @@ zparser_conv_short(region_type *region, const char *text)
 	uint16_t *r = NULL;
 	uint16_t value;
 	char *end;
-   
+
 	value = htons((uint16_t) strtol(text, &end, 10));
 	if (*end != '\0') {
 		zc_error_prev_line("integer value is expected");
@@ -285,7 +305,7 @@ zparser_conv_byte(region_type *region, const char *text)
 	uint16_t *r = NULL;
 	uint8_t value;
 	char *end;
-   
+
 	value = (uint8_t) strtol(text, &end, 10);
 	if (*end != '\0') {
 		zc_error_prev_line("integer value is expected");
@@ -300,7 +320,7 @@ zparser_conv_algorithm(region_type *region, const char *text)
 {
 	const lookup_table_type *alg;
 	uint8_t id;
-	
+
 	alg = lookup_by_name(dns_algorithms, text);
 	if (alg) {
 		id = (uint8_t) alg->id;
@@ -322,7 +342,7 @@ zparser_conv_certificate_type(region_type *region, const char *text)
 	/* convert a algoritm string to integer */
 	const lookup_table_type *type;
 	uint16_t id;
-	
+
 	type = lookup_by_name(dns_certificate_types, text);
 	if (type) {
 		id = htons((uint16_t) type->id);
@@ -370,7 +390,7 @@ uint16_t *
 zparser_conv_text(region_type *region, const char *text, size_t len)
 {
 	uint16_t *r = NULL;
-	
+
 	if (len > 255) {
 		zc_error_prev_line("text string is longer than 255 characters,"
 				   " try splitting it into multiple parts");
@@ -384,6 +404,18 @@ zparser_conv_text(region_type *region, const char *text, size_t len)
 	return r;
 }
 
+uint16_t *
+zparser_conv_dns_name(region_type *region, const uint8_t* name, size_t len)
+{
+	uint16_t* r = NULL;
+	uint8_t* p = NULL;
+	r = alloc_rdata(region, len);
+	p = (uint8_t *) (r + 1);
+	memcpy(p, name, len);
+
+	return r;
+}
+
 uint16_t *
 zparser_conv_b32(region_type *region, const char *b32)
 {
@@ -479,7 +511,7 @@ zparser_conv_nsec(region_type *region,
 
 	window_max = 1 + (nsec_highest_rcode / 256);
 
-	/* used[i] is the i-th window included in the nsec 
+	/* used[i] is the i-th window included in the nsec
 	 * size[used[0]] is the size of window 0
 	 */
 
@@ -502,7 +534,7 @@ zparser_conv_nsec(region_type *region,
 	for (i = 0; i < window_count; ++i) {
 		total_size += sizeof(uint16_t) + size[used[i]];
 	}
-	
+
 	r = alloc_rdata(region, total_size);
 	ptr = (uint8_t *) (r + 1);
 
@@ -547,7 +579,7 @@ static unsigned int poweroften[10] = {1, 10, 100, 1000, 10000, 100000,
  * Sets the given pointer to the last used character.
  *
  */
-static uint8_t 
+static uint8_t
 precsize_aton (char *cp, char **endptr)
 {
 	unsigned int mval = 0, cmval = 0;
@@ -555,14 +587,14 @@ precsize_aton (char *cp, char **endptr)
 	int exponent;
 	int mantissa;
 
-	while (isdigit(*cp))
+	while (isdigit((int)*cp))
 		mval = mval * 10 + hexdigit_to_int(*cp++);
 
 	if (*cp == '.') {	/* centimeters */
 		cp++;
-		if (isdigit(*cp)) {
+		if (isdigit((int)*cp)) {
 			cmval = hexdigit_to_int(*cp++) * 10;
-			if (isdigit(*cp)) {
+			if (isdigit((int)*cp)) {
 				cmval += hexdigit_to_int(*cp++);
 			}
 		}
@@ -612,14 +644,13 @@ zparser_conv_loc(region_type *region, char *str)
 	int deg, min, secs;	/* Secs is stored times 1000.  */
 	uint32_t lat = 0, lon = 0, alt = 0;
 	/* encoded defaults: version=0 sz=1m hp=10000m vp=10m */
-	uint8_t vszhpvp[4] = {0, 0x12, 0x16, 0x13}; 
+	uint8_t vszhpvp[4] = {0, 0x12, 0x16, 0x13};
 	char *start;
 	double d;
-			
 
 	for(;;) {
 		deg = min = secs = 0;
-		
+
 		/* Degrees */
 		if (*str == '\0') {
 			zc_error_prev_line("unexpected end of LOC data");
@@ -628,25 +659,25 @@ zparser_conv_loc(region_type *region, char *str)
 
 		if (!parse_int(str, &str, &deg, "degrees", 0, 180))
 			return NULL;
-		if (!isspace(*str)) {
+		if (!isspace((int)*str)) {
 			zc_error_prev_line("space expected after degrees");
 			return NULL;
 		}
 		++str;
-		
+
 		/* Minutes? */
-		if (isdigit(*str)) {
+		if (isdigit((int)*str)) {
 			if (!parse_int(str, &str, &min, "minutes", 0, 60))
 				return NULL;
-			if (!isspace(*str)) {
+			if (!isspace((int)*str)) {
 				zc_error_prev_line("space expected after minutes");
 				return NULL;
 			}
 			++str;
 		}
-		
+
 		/* Seconds? */
-		if (isdigit(*str)) {
+		if (isdigit((int)*str)) {
 			start = str;
 			if (!parse_int(str, &str, &i, "seconds", 0, 60)) {
 				return NULL;
@@ -656,7 +687,7 @@ zparser_conv_loc(region_type *region, char *str)
 				return NULL;
 			}
 
-			if (!isspace(*str)) {
+			if (!isspace((int)*str)) {
 				zc_error_prev_line("space expected after seconds");
 				return NULL;
 			}
@@ -672,7 +703,7 @@ zparser_conv_loc(region_type *region, char *str)
 			secs = (int) (d * 1000.0 + 0.5);
 			++str;
 		}
-		
+
 		switch(*str) {
 		case 'N':
 		case 'n':
@@ -695,11 +726,11 @@ zparser_conv_loc(region_type *region, char *str)
 			return NULL;
 		}
 		++str;
-		
+
 		if (lat != 0 && lon != 0)
 			break;
 
-		if (!isspace(*str)) {
+		if (!isspace((int)*str)) {
 			zc_error_prev_line("space expected after latitude/longitude");
 			return NULL;
 		}
@@ -712,7 +743,7 @@ zparser_conv_loc(region_type *region, char *str)
 		return NULL;
 	}
 
-	if (!isspace(*str)) {
+	if (!isspace((int)*str)) {
 		zc_error_prev_line("space expected before altitude");
 		return NULL;
 	}
@@ -726,7 +757,7 @@ zparser_conv_loc(region_type *region, char *str)
 	}
 
 	/* Meters of altitude... */
-	(void)strtol(str, &str, 10);
+	(void) strtol(str, &str, 10);
 	switch(*str) {
 	case ' ':
 	case '\0':
@@ -736,7 +767,7 @@ zparser_conv_loc(region_type *region, char *str)
 		if (!parse_int(str + 1, &str, &i, "altitude fraction", 0, 99)) {
 			return NULL;
 		}
-		if (!isspace(*str) && *str != '\0' && *str != 'm') {
+		if (!isspace((int)*str) && *str != '\0' && *str != 'm') {
 			zc_error_prev_line("altitude fraction must be a number");
 			return NULL;
 		}
@@ -745,25 +776,25 @@ zparser_conv_loc(region_type *region, char *str)
 		zc_error_prev_line("altitude must be expressed in meters");
 		return NULL;
 	}
-	if (!isspace(*str) && *str != '\0')
+	if (!isspace((int)*str) && *str != '\0')
 		++str;
 
 	if (sscanf(start, "%lf", &d) != 1) {
 		zc_error_prev_line("error parsing altitude");
 	}
-	
+
 	alt = (uint32_t) (10000000.0 + d * 100 + 0.5);
 
-	if (!isspace(*str) && *str != '\0') {
+	if (!isspace((int)*str) && *str != '\0') {
 		zc_error_prev_line("unexpected character after altitude");
 		return NULL;
 	}
 
 	/* Now parse size, horizontal precision and vertical precision if any */
-	for(i = 1; isspace(*str) && i <= 3; i++) {
+	for(i = 1; isspace((int)*str) && i <= 3; i++) {
 		vszhpvp[i] = precsize_aton(str + 1, &str);
 
-		if (!isspace(*str) && *str != '\0') {
+		if (!isspace((int)*str) && *str != '\0') {
 			zc_error_prev_line("invalid size or precision");
 			return NULL;
 		}
@@ -802,7 +833,7 @@ zparser_conv_apl_rdata(region_type *region, char *str)
 	uint8_t *t;
 	char *end;
 	long p;
-	
+
 	if (!colon) {
 		zc_error("address family separator is missing");
 		return NULL;
@@ -814,7 +845,7 @@ zparser_conv_apl_rdata(region_type *region, char *str)
 
 	*colon = '\0';
 	*slash = '\0';
-	
+
 	if (*str == '!') {
 		negated = 1;
 		++str;
@@ -848,7 +879,7 @@ zparser_conv_apl_rdata(region_type *region, char *str)
 	while (length > 0 && address[length - 1] == 0)
 		--length;
 
-	
+
 	p = strtol(slash + 1, &end, 10);
 	if (p < 0 || p > maximum_prefix) {
 		zc_error("prefix not in the range 0 .. %d", maximum_prefix);
@@ -863,7 +894,7 @@ zparser_conv_apl_rdata(region_type *region, char *str)
 		    + length);
 	r = alloc_rdata(region, rdlength);
 	t = (uint8_t *) (r + 1);
-	
+
 	memcpy(t, &address_family, sizeof(address_family));
 	t += sizeof(address_family);
 	memcpy(t, &prefix, sizeof(prefix));
@@ -878,28 +909,28 @@ zparser_conv_apl_rdata(region_type *region, char *str)
 	return r;
 }
 
-/* 
+/*
  * Below some function that also convert but not to wireformat
  * but to "normal" (int,long,char) types
  */
 
-int32_t
-zparser_ttl2int(const char *ttlstr)
+uint32_t
+zparser_ttl2int(const char *ttlstr, int* error)
 {
 	/* convert a ttl value to a integer
 	 * return the ttl in a int
 	 * -1 on error
 	 */
 
-	int32_t ttl;
+	uint32_t ttl;
 	const char *t;
 
 	ttl = strtottl(ttlstr, &t);
 	if (*t != 0) {
 		zc_error_prev_line("invalid TTL value: %s",ttlstr);
-		ttl = -1;
+		*error = 1;
 	}
-    
+
 	return ttl;
 }
 
@@ -954,7 +985,7 @@ parse_unknown_rdata(uint16_t type, uint16_t *wireformat)
 		zc_error_prev_line("bad unknown RDATA");
 		return;
 	}
-	
+
 	for (i = 0; i < rdata_count; ++i) {
 		if (rdata_atom_is_domain(type, i)) {
 			zadd_rdata_domain(rdatas[i].domain);
@@ -978,10 +1009,10 @@ static int
 zrdatacmp(uint16_t type, rr_type *a, rr_type *b)
 {
 	int i = 0;
-	
+
 	assert(a);
 	assert(b);
-	
+
 	/* One is shorter than another */
 	if (a->rdata_count != b->rdata_count)
 		return 1;
@@ -1045,7 +1076,7 @@ zone_open(const char *filename, uint32_t ttl, uint16_t klass,
 }
 
 
-void 
+void
 set_bitnsec(uint8_t bits[NSEC_WINDOW_COUNT][NSEC_WINDOW_BITS_SIZE],
 	    uint16_t index)
 {
@@ -1055,7 +1086,7 @@ set_bitnsec(uint8_t bits[NSEC_WINDOW_COUNT][NSEC_WINDOW_BITS_SIZE],
 	 */
 	uint8_t window = index / 256;
 	uint8_t bit = index % 256;
-		
+
 	bits[window][bit / 8] |= (1 << (7 - bit % 8));
 }
 
@@ -1079,7 +1110,7 @@ process_rr(void)
 	int i;
 	rrtype_descriptor_type *descriptor
 		= rrtype_descriptor_by_type(rr->type);
-	
+
 	/* We only support IN class */
 	if (rr->klass != CLASS_IN) {
 		zc_error_prev_line("only class IN is supported");
@@ -1094,7 +1125,7 @@ process_rr(void)
 		zc_error_prev_line("maximum rdata length exceeds %d octets", MAX_RDLENGTH);
 		return 0;
 	}
-		     
+
 	if (rr->type == TYPE_SOA) {
 		/*
 		 * This is a SOA record, start a new zone or continue
@@ -1125,7 +1156,7 @@ process_rr(void)
 
 			rr->owner->is_apex = 1;
 		}
-		
+
 		/* parser part */
 		parser->current_zone = zone;
 	}
@@ -1146,7 +1177,7 @@ process_rr(void)
 		rrset->rr_count = 1;
 		rrset->rrs = (rr_type *) xalloc(sizeof(rr_type));
 		rrset->rrs[0] = *rr;
-			
+
 		region_add_cleanup(parser->region, cleanup_rrset, rrset);
 
 		/* Add it */
@@ -1197,7 +1228,7 @@ process_rr(void)
 		rrset->zone->is_secure = 1;
 	}
 #endif
-	
+
 	/* Check we have SOA */
 	/* [XXX] this is dead code */
 	if (zone->soa_rrset == NULL) {
@@ -1220,7 +1251,7 @@ process_rr(void)
 		zone->ns_rrset = rrset;
 	}
 	if (vflag > 1 && totalrrs > 0 && (totalrrs % progress == 0)) {
-		printf("%ld\n", totalrrs);
+		fprintf(stdout, "%ld\n", totalrrs);
 	}
 	++totalrrs;
 	return 1;
@@ -1245,7 +1276,7 @@ domain_find_rrset_any(domain_type *domain, uint16_t type)
 /*
  * Check for DNAME type. Nothing is allowed below it
  */
-static void 
+static void
 check_dname(namedb_type* db)
 {
 	domain_type* domain;
@@ -1288,7 +1319,7 @@ zone_read(const char *name, const char *zonefile, nsd_options_t* nsd_options)
 		zc_error("incorrect zone name '%s'", name);
 		return;
 	}
-	
+
 #ifndef ROOT_SERVER
 	/* Is it a root zone? Are we a root server then? Idiot proof. */
 	if (dname->label_count == 1) {
@@ -1304,7 +1335,7 @@ zone_read(const char *name, const char *zonefile, nsd_options_t* nsd_options)
 			zone_options_t* zopt = zone_options_find(nsd_options, dname);
 			if(zopt && zone_is_slave(zopt)) {
 				zc_warning("slave zone %s with no zonefile '%s'(%s) will "
-					"force zone transfer.", 
+					"force zone transfer.",
 					name, zonefile, strerror(errno));
 				return;
 			}
@@ -1318,7 +1349,7 @@ zone_read(const char *name, const char *zonefile, nsd_options_t* nsd_options)
 	yyparse();
 
 	/* check if zone file contained a correct SOA record */
-	if (parser->current_zone && parser->current_zone->soa_rrset 
+	if (parser->current_zone && parser->current_zone->soa_rrset
 		&& parser->current_zone->soa_rrset->rr_count!=0)
 	{
 		if(dname_compare(domain_dname(
@@ -1332,13 +1363,13 @@ zone_read(const char *name, const char *zonefile, nsd_options_t* nsd_options)
 	}
 
 	fclose(yyin);
-	
+
 	fflush(stdout);
 	totalerrors += parser->errors;
 	parser->filename = NULL;
 }
 
-static void 
+static void
 usage (void)
 {
 #ifndef NDEBUG
@@ -1347,7 +1378,7 @@ usage (void)
 	fprintf(stderr, "usage: zonec [-v|-h|-C] [-c configfile] [-o origin] [-d directory] [-f database] [-z zonefile]\n\n");
 #endif
 	fprintf(stderr, "\tNSD zone compiler, creates database from zone files.\n");
-	fprintf(stderr, "\tVersion %s. Report bugs to <%s>.\n\n", 
+	fprintf(stderr, "\tVersion %s. Report bugs to <%s>.\n\n",
 		PACKAGE_VERSION, PACKAGE_BUGREPORT);
 	fprintf(stderr, "\t-v\tBe more verbose.\n");
 	fprintf(stderr, "\t-h\tPrint this help information.\n");
@@ -1366,7 +1397,7 @@ usage (void)
 extern char *optarg;
 extern int optind;
 
-int 
+int
 main (int argc, char **argv)
 {
 	struct namedb *db;
@@ -1378,7 +1409,7 @@ main (int argc, char **argv)
 	const char* zonesdir = NULL;
 	const char* singlefile = NULL;
 	nsd_options_t* nsd_options = NULL;
-	
+
 	log_init("zonec");
 
 	global_region = region_create(xalloc, free);
@@ -1458,11 +1489,16 @@ main (int argc, char **argv)
 
 	/* Create the database */
 	if ((db = namedb_new(dbfile)) == NULL) {
-		fprintf(stderr, "zonec: error creating the database: %s\n", dbfile);
+		fprintf(stderr, "zonec: error creating the database (%s): %s\n",
+			dbfile, strerror(errno));
 		exit(1);
 	}
 
 	parser = zparser_create(global_region, rr_region, db);
+	if (!parser) {
+		fprintf(stderr, "zonec: error creating the parser\n");
+		exit(1);
+	}
 
 	/* Unique pointers used to mark errors.	 */
 	error_dname = (dname_type *) region_alloc(global_region, 0);
@@ -1472,7 +1508,7 @@ main (int argc, char **argv)
 		/*
 		 * Read a single zone file with the specified origin
 		 */
-		if(!singlefile || !origin) {
+		if(!singlefile) {
 			fprintf(stderr, "zonec: must have -z zonefile when reading single zone.\n");
 			exit(1);
 		}
@@ -1481,10 +1517,10 @@ main (int argc, char **argv)
 			exit(1);
 		}
 		if (vflag > 0)
-			fprintf(stderr, "zonec: reading zone \"%s\".\n", origin);
+			fprintf(stdout, "zonec: reading zone \"%s\".\n", origin);
 		zone_read(origin, singlefile, nsd_options);
 		if (vflag > 0)
-			fprintf(stderr, "zonec: processed %ld RRs in \"%s\".\n", totalrrs, origin);
+			fprintf(stdout, "zonec: processed %ld RRs in \"%s\".\n", totalrrs, origin);
 	} else {
 		zone_options_t* zone;
 		if(!nsd_options) {
@@ -1495,11 +1531,11 @@ main (int argc, char **argv)
 		RBTREE_FOR(zone, zone_options_t*, nsd_options->zone_options)
 		{
 			if (vflag > 0)
-				fprintf(stderr, "zonec: reading zone \"%s\".\n",
+				fprintf(stdout, "zonec: reading zone \"%s\".\n",
 					zone->name);
 			zone_read(zone->name, zone->zonefile, nsd_options);
 			if (vflag > 0)
-				fprintf(stderr,
+				fprintf(stdout,
 					"zonec: processed %ld RRs in \"%s\".\n",
 					totalrrs, zone->name);
 			totalrrs = 0;
@@ -1508,17 +1544,17 @@ main (int argc, char **argv)
 	check_dname(db);
 
 #ifndef NDEBUG
-	fprintf(stderr, "global_region: ");
+	fprintf(stdout, "global_region: ");
 	region_dump_stats(global_region, stderr);
-	fprintf(stderr, "\n");
-	fprintf(stderr, "db->region: ");
+	fprintf(stdout, "\n");
+	fprintf(stdout, "db->region: ");
 	region_dump_stats(db->region, stderr);
-	fprintf(stderr, "\n");
+	fprintf(stdout, "\n");
 #endif /* NDEBUG */
-	
+
 	/* Close the database */
 	if (namedb_save(db) != 0) {
-		fprintf(stderr, "zonec: error saving the database: %s\n", strerror(errno));
+		fprintf(stderr, "zonec: error writing the database (%s): %s\n", db->filename, strerror(errno));
 		namedb_discard(db);
 		exit(1);
 	}
@@ -1528,11 +1564,11 @@ main (int argc, char **argv)
 		fprintf(stderr, "\nzonec: done with %ld errors.\n",
 			totalerrors);
 	}
-	
+
 	/* Disable this to save some time.  */
 #if 0
 	region_destroy(global_region);
 #endif
-	
+
 	return totalerrors ? 1 : 0;
 }
